/* 
 * File   : MP3DecoderAbstraction.h
 * Author : Serhiy Tkachenko <Serhiy.Tkachenko@tomtom.com>
 * Desc.  : C++ like abstraction for MP3 dencoder definition
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __MP3_DECODER_ABSTRACTION_H__
#define __MP3_DECODER_ABSTRACTION_H__

#include "MadProducer.h"

/** \brief Abstract MP3 decoder definition
*/
typedef
 struct  
    { 
     /** \brief Initializes MP3 decoder

     Returns 0 in case of success
     
     \param aInputFile   Open MP3 file descriptor
     \param aBufferSize  Internal buffer size, should be 1024 bytes aligned
     \param aFlags       Creation flags, refer to \a MadProducer.h 
     */
     int   (* iMP3Init )( int aInputFile, int aBufferSize, unsigned long aFlags );

     /** \brief Decodes given count of bytes from the stream

     \param Count Number of bytes to decode. BufferiSize must be in multiples of
                  this value
     Returns NULL in case of error or end of stream
     */
     unsigned char* (* iMP3Decode )( int Count );
     
     void  (* iMP3Finish )();
     
    } TMP3Decoder;

typedef enum 
{ 
  KMADMP3 = 0
} EMP3DecoderType;

/** \brief SBC encoder creating function 

 General accepted sample parameters:
 16-bit, stereo, 44.1kHz

 TomTom encoder accepts only input samples in MAD fixed point format
 Indian encoder accepts only normal 16-bit samples
*/
TMP3Decoder* CreateMP3Decoder( EMP3DecoderType aType );
void         DestroyMP3Decoder();

#endif // __MP3_DECODER_ABSTRACTION_H__

