/* 
 * File   : SBCEncoderAbstraction.h
 * Author : Serhiy Tkachenko <Serhiy.Tkachenko@tomtom.com>
 * Desc.  : C++ like abstraction for SBC encoders definition
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 */


#ifndef __SBC_ENCODER_ABSTRACTION_H__
#define __SBC_ENCODER_ABSTRACTION_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <string.h>

#define SBC_NULL        0x00000001

/** \brief Sbc parameter structure
*/
struct sbc_struct {
        unsigned long flags;

        int rate;
        int channels;
        int blocks;
        int subbands;
        int bitpool;
        int allocation_method;
        int channel_mode;

        void *data; // Should be set by application, is not allocated automatically !!!
        int  size;  
        int  len;
        int  input_len;

        unsigned long duration;
	double durationDouble;

        void *priv;
};

typedef struct sbc_struct sbc_t;

/** \brief Return values for SBCEncoderInputType
*/
#define SBC_IN_PCM     0  
#define SBC_IN_PCMS    1
#define SBC_IN_MAD     2


/** \brief Abstract SBC encoder definition
*/
typedef
 struct  
    { 
    

     int  (* iSBCInputType)(void);
     /** \brief Initialization

     Returns 0 in case of success
     */
     int  (* iSBCInit )( sbc_t *sbc, unsigned long flags, int aBitPool );

     /** \brief Encodes a new SBC frame

     \param sbc     Pointer to SBC data structure
     \param data    Input PCM samples
     \param size    Size of input buffer. Value of 1024 is recommended.

     Returns the number of PCM BYTES used.

     The encoded frame is stored in \a sbc->data member and 
     the length of this frame in \a sbc->len.
     In case of error \a sbc->len should be set to a negative value
     */
     int  (* iSBCEncode)( sbc_t *sbc, void *data, int count );
     void (* iSBCFinish )( sbc_t *sbc );
     
    } TSBCEncoder;


#ifdef __cplusplus
}
#endif

#endif // __SBC_ENCODER_ABSTRACTION_H__

