
#include <assert.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#include "SBCConfig.h"
#include "SBCDef.h"
#include "fixed.h"
#include "sbc_tables.h"
#include "sbc_tables_f.h"

/*
static void CheckS( char* aPreffix, mad_fixed_t* aSubbandSamples, double* tmpTestSB, mad_fixed_t* Y, double* TestY, int tmpRound )
{
  int i, j, b;

  b= 0;
  for( i = 0; i < 8; ++i )
  {
   if( fabs( mad_f_todouble( aSubbandSamples[ i ] ) * ( 1L << PCM_SHIFT ) - 
       tmpTestSB[ i ] ) > 0.5 )
   {
    fprintf( stderr, "%s R %d SB[ %d ] New %lf Old %lf\n", aPreffix, tmpRound, i,
        mad_f_todouble( aSubbandSamples[ i ] ) * ( 1L << PCM_SHIFT ),
        tmpTestSB[ i ] );

    for( j = 0; j < 16; ++j )
    {
     fprintf( stderr, "%d Ana New %lf Old %lf, Y New %lf Old %lf\n",
              j, mad_f_todouble( anamatrix8_f[ i ][ j ] ),
              anamatrix8[ i ][ j ],
              mad_f_todouble( Y[ j ] ) * ( 1L << PCM_SHIFT ),
              TestY[ j ] );
    }

    fprintf( stderr, "\n\n" );
    b = 1;
   }

   if( b )
           exit( 0 );
  }
}
*/
/*
 Polyphase filter
 PCM samples format: mad_fixed_t 3.28
 1 bit for sign, 3 bits for integer part and 28 bits for fractional part

 S[ i ] should be shifted << to be compatible with the rest (see sbc_analys_eight()
 in sbc.c)

 Possible glues for optimisation/increasing sound quality:
 1. Keep Y 64 bit, remove MAD_F_MLZ
 2. Calculate S using an optimized manner (see floating point code)

 At global level:
 1. Keep S 64 bit (might be difficult beacuse the rest of code is crapy)
 2. Optimize scale factors calculation (sbc_pack_frame() in sbc.c)
    Unroll the loops
    Use fixed jump tables for it
 3. Make the quantization better. Assignment to frame->audio_sample[ blk ][ ch ][ sb ]
     (sbc_pack_frame() in sbc.c)

*/
void SBCAnalyze8( SBCSampleList* aSamples, mad_fixed_t* aSubbandSamples )
{
 assert( NULL != aSamples );
 assert( NULL != aSubbandSamples );

 //static int tmpRound = 0;

          mad_fixed_t     Y[ 16 ];
// register sbc_fixed_t     tmpVal;
//          double          Y[ 16 ];
//          double          TestY[ 16 ];
//          double          tmpTestSB[ 8 ];
//          int             i, j;
// register mad_fixed_t     tmpVal, tt;
// register double          tmpDVal;
 register mad_fixed64hi_t hi;
 register mad_fixed64lo_t lo;
          mad_fixed64hi_t hi_s[ 8 ];
          mad_fixed64lo_t lo_s[ 8 ];
          mad_fixed_t     ty04, ty13, ty515, ty614, ty713, ty812, ty911;
//             int          b;

// memset( tmpTestSB, 0, 8 * sizeof( double ) );
// memset( TestY, 0, sizeof( TestY ) );
// memset( Y, 0, sizeof( Y ) );
 
 // This is an optimized version of Y calculation below
 // 2 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 16 ], 
                    aSamples->m_pSamples[ 16 ] - aSamples->m_pSamples[ 64 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 32 ],
                    aSamples->m_pSamples[ 32 ] - aSamples->m_pSamples[ 48 ] );
 Y[ 0 ] = MAD_F_MLZ( hi, lo );

  // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 1 ],  aSamples->m_pSamples[ 1 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 17 ], aSamples->m_pSamples[ 17 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 33 ], aSamples->m_pSamples[ 33 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 49 ], aSamples->m_pSamples[ 49 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 65 ], aSamples->m_pSamples[ 65 ] );
 Y[ 1 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 2 ],  aSamples->m_pSamples[ 2 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 18 ], aSamples->m_pSamples[ 18 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 34 ], aSamples->m_pSamples[ 34 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 50 ], aSamples->m_pSamples[ 50 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 66 ], aSamples->m_pSamples[ 66 ] );
 //// Not necessary, because of next hi_s and lo_s initialisation
 Y[ 2 ] = MAD_F_MLZ( hi, lo );  

 // Subbands initialisation 
 /*aSubbandSamples[ 0 ] = aSubbandSamples[ 1 ] = aSubbandSamples[ 2 ] = aSubbandSamples[ 3 ] =
 aSubbandSamples[ 4 ] = aSubbandSamples[ 5 ] = aSubbandSamples[ 6 ] = aSubbandSamples[ 7 ] = 
  MAD_F_MLZ( hi, lo ); */

 hi_s[ 0 ] = hi_s[ 1 ] = hi_s[ 2 ] = hi_s[ 3 ] = hi_s[ 4 ] = 
 hi_s[ 5 ] = hi_s[ 6 ] = hi_s[ 7 ] = hi;
 lo_s[ 0 ] = lo_s[ 1 ] = lo_s[ 2 ] = lo_s[ 3 ] = lo_s[ 4 ] = 
 lo_s[ 5 ] = lo_s[ 6 ] = lo_s[ 7 ] = lo;

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 3 ],  aSamples->m_pSamples[ 3 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 19 ], aSamples->m_pSamples[ 19 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 35 ], aSamples->m_pSamples[ 35 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 51 ], aSamples->m_pSamples[ 51 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 67 ], aSamples->m_pSamples[ 67 ] );
 Y[ 3 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 4 ],  aSamples->m_pSamples[ 4 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 20 ], aSamples->m_pSamples[ 20 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 36 ], aSamples->m_pSamples[ 36 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 52 ], aSamples->m_pSamples[ 52 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 68 ], aSamples->m_pSamples[ 68 ] );
 Y[ 4 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 5 ],  aSamples->m_pSamples[ 5 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 21 ], aSamples->m_pSamples[ 21 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 37 ], aSamples->m_pSamples[ 37 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 53 ], aSamples->m_pSamples[ 53 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 69 ], aSamples->m_pSamples[ 69 ] );
 Y[ 5 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 6 ], aSamples->m_pSamples[ 6 ]   );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 22 ], aSamples->m_pSamples[ 22 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 38 ], aSamples->m_pSamples[ 38 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 54 ], aSamples->m_pSamples[ 54 ] );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 70 ], aSamples->m_pSamples[ 70 ] );
 Y[ 6 ] = MAD_F_MLZ( hi, lo );

  // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 7 ] , aSamples->m_pSamples[ 7 ]   );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 23 ], aSamples->m_pSamples[ 23 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 39 ], aSamples->m_pSamples[ 39 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 55 ], aSamples->m_pSamples[ 55 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 71 ], aSamples->m_pSamples[ 71 ]  );
 Y[ 7 ] = MAD_F_MLZ( hi, lo );

 // 3 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 8 ] , ( aSamples->m_pSamples[ 8 ] + aSamples->m_pSamples[ 72 ] )  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 24 ], ( aSamples->m_pSamples[ 24 ] + aSamples->m_pSamples[ 56 ] )  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 40 ], aSamples->m_pSamples[ 40 ] );
 Y[ 8 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 9 ] , aSamples->m_pSamples[ 9 ]   );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 25 ], aSamples->m_pSamples[ 25 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 41 ], aSamples->m_pSamples[ 41 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 57 ], aSamples->m_pSamples[ 57 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 73 ], aSamples->m_pSamples[ 73 ]  );
 Y[ 9 ] = MAD_F_MLZ( hi, lo );

 // 5 mult Y[ 10 ] is not used

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 11 ], aSamples->m_pSamples[ 11 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 27 ], aSamples->m_pSamples[ 27 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 43 ], aSamples->m_pSamples[ 43 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 59 ], aSamples->m_pSamples[ 59 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 75 ], aSamples->m_pSamples[ 75 ]  );
 Y[ 11 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 12 ], aSamples->m_pSamples[ 12 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 28 ], aSamples->m_pSamples[ 28 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 44 ], aSamples->m_pSamples[ 44 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 60 ], aSamples->m_pSamples[ 60 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 76 ], aSamples->m_pSamples[ 76 ]  );
 Y[ 12 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 13 ], aSamples->m_pSamples[ 13 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 29 ], aSamples->m_pSamples[ 29 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 45 ], aSamples->m_pSamples[ 45 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 61 ], aSamples->m_pSamples[ 61 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 77 ], aSamples->m_pSamples[ 77 ]  );
 Y[ 13 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 14 ], aSamples->m_pSamples[ 14 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 30 ], aSamples->m_pSamples[ 30 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 46 ], aSamples->m_pSamples[ 46 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 62 ], aSamples->m_pSamples[ 62 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 78 ], aSamples->m_pSamples[ 78 ]  );
 Y[ 14 ] = MAD_F_MLZ( hi, lo );

 // 5 mult
 MAD_F_ML0( hi, lo, sbc_proto_8_80_f[ 15 ], aSamples->m_pSamples[ 15 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 31 ], aSamples->m_pSamples[ 31 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 47 ], aSamples->m_pSamples[ 47 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 63 ], aSamples->m_pSamples[ 63 ]  );
 MAD_F_MLA( hi, lo, sbc_proto_8_80_f[ 79 ], aSamples->m_pSamples[ 79 ]  );
 Y[ 15 ] = MAD_F_MLZ( hi, lo );
/*
 // 2 mult
 TestY[ 0 ] = sbc_proto_8_80[ 16 ] * ( aSamples->m_pTestSamples[ 16 ] - aSamples->m_pTestSamples[ 64 ] ) +
          sbc_proto_8_80[ 32 ] * ( aSamples->m_pTestSamples[ 32 ] - aSamples->m_pTestSamples[ 48 ] );
 
 // 5 mult
 TestY[ 1 ] = sbc_proto_8_80[ 1 ]  * aSamples->m_pTestSamples[ 1 ]  +
          sbc_proto_8_80[ 17 ] * aSamples->m_pTestSamples[ 17 ] +
          sbc_proto_8_80[ 33 ] * aSamples->m_pTestSamples[ 33 ] +
          sbc_proto_8_80[ 49 ] * aSamples->m_pTestSamples[ 49 ] +
          sbc_proto_8_80[ 65 ] * aSamples->m_pTestSamples[ 65 ];

 // 5 mult
 TestY[ 2 ] = sbc_proto_8_80[ 2 ]  * aSamples->m_pTestSamples[ 2 ]  +
          sbc_proto_8_80[ 18 ] * aSamples->m_pTestSamples[ 18 ] +
          sbc_proto_8_80[ 34 ] * aSamples->m_pTestSamples[ 34 ] +
          sbc_proto_8_80[ 50 ] * aSamples->m_pTestSamples[ 50 ] +
          sbc_proto_8_80[ 66 ] * aSamples->m_pTestSamples[ 66 ];

 // 5 mult
 TestY[ 3 ] = sbc_proto_8_80[ 3 ]  * aSamples->m_pTestSamples[ 3 ]  +
          sbc_proto_8_80[ 19 ] * aSamples->m_pTestSamples[ 19 ] +
          sbc_proto_8_80[ 35 ] * aSamples->m_pTestSamples[ 35 ] +
          sbc_proto_8_80[ 51 ] * aSamples->m_pTestSamples[ 51 ] +
          sbc_proto_8_80[ 67 ] * aSamples->m_pTestSamples[ 67 ];

 // 5 mult
 TestY[ 4 ] = sbc_proto_8_80[ 4 ]  * aSamples->m_pTestSamples[ 4 ]  +
          sbc_proto_8_80[ 20 ] * aSamples->m_pTestSamples[ 20 ] +
          sbc_proto_8_80[ 36 ] * aSamples->m_pTestSamples[ 36 ] +
          sbc_proto_8_80[ 52 ] * aSamples->m_pTestSamples[ 52 ] +
          sbc_proto_8_80[ 68 ] * aSamples->m_pTestSamples[ 68 ];

 // 5 mult
 TestY[ 5 ] = sbc_proto_8_80[ 5 ]  * aSamples->m_pTestSamples[ 5 ]  +
          sbc_proto_8_80[ 21 ] * aSamples->m_pTestSamples[ 21 ] +
          sbc_proto_8_80[ 37 ] * aSamples->m_pTestSamples[ 37 ] +
          sbc_proto_8_80[ 53 ] * aSamples->m_pTestSamples[ 53 ] +
          sbc_proto_8_80[ 69 ] * aSamples->m_pTestSamples[ 69 ];

 // 5 mult
 TestY[ 6 ] = sbc_proto_8_80[ 6 ]  * aSamples->m_pTestSamples[ 6 ]  +
          sbc_proto_8_80[ 22 ] * aSamples->m_pTestSamples[ 22 ] +
          sbc_proto_8_80[ 38 ] * aSamples->m_pTestSamples[ 38 ] +
          sbc_proto_8_80[ 54 ] * aSamples->m_pTestSamples[ 54 ] +
          sbc_proto_8_80[ 70 ] * aSamples->m_pTestSamples[ 70 ];

  // 5 mult
 TestY[ 7 ] = sbc_proto_8_80[ 7 ]  * aSamples->m_pTestSamples[ 7 ]  +
          sbc_proto_8_80[ 23 ] * aSamples->m_pTestSamples[ 23 ] +
          sbc_proto_8_80[ 39 ] * aSamples->m_pTestSamples[ 39 ] +
          sbc_proto_8_80[ 55 ] * aSamples->m_pTestSamples[ 55 ] +
          sbc_proto_8_80[ 71 ] * aSamples->m_pTestSamples[ 71 ];

 // 3 mult
 TestY[ 8 ] = sbc_proto_8_80[ 8 ]  * ( aSamples->m_pTestSamples[ 8 ] + aSamples->m_pTestSamples[ 72 ] ) +
          sbc_proto_8_80[ 24 ] * ( aSamples->m_pTestSamples[ 24 ] + aSamples->m_pTestSamples[ 56 ] ) +
          sbc_proto_8_80[ 40 ] * aSamples->m_pTestSamples[ 40 ];

 // 5 mult
 TestY[ 9 ] = sbc_proto_8_80[ 9 ]  * aSamples->m_pTestSamples[ 9 ]  +
          sbc_proto_8_80[ 25 ] * aSamples->m_pTestSamples[ 25 ] +
          sbc_proto_8_80[ 41 ] * aSamples->m_pTestSamples[ 41 ] +
          sbc_proto_8_80[ 57 ] * aSamples->m_pTestSamples[ 57 ] +
          sbc_proto_8_80[ 73 ] * aSamples->m_pTestSamples[ 73 ];

 // 5 mult TestY[ 10 ] is not used
// TestY[ 10 ] = sbc_proto_8_80[ 10 ] * aSamples->m_pTestSamples[ 10 ] +
//           sbc_proto_8_80[ 26 ] * aSamples->m_pTestSamples[ 26 ] +
//           sbc_proto_8_80[ 42 ] * aSamples->m_pTestSamples[ 42 ] +
//           sbc_proto_8_80[ 58 ] * aSamples->m_pTestSamples[ 58 ] +
//           sbc_proto_8_80[ 74 ] * aSamples->m_pTestSamples[ 74 ]; 

 // 5 mult
 TestY[ 11 ] = sbc_proto_8_80[ 11 ] * aSamples->m_pTestSamples[ 11 ] +
           sbc_proto_8_80[ 27 ] * aSamples->m_pTestSamples[ 27 ] +
           sbc_proto_8_80[ 43 ] * aSamples->m_pTestSamples[ 43 ] +
           sbc_proto_8_80[ 59 ] * aSamples->m_pTestSamples[ 59 ] +
           sbc_proto_8_80[ 75 ] * aSamples->m_pTestSamples[ 75 ];

 // 5 mult
 TestY[ 12 ] = sbc_proto_8_80[ 12 ] * aSamples->m_pTestSamples[ 12 ] +
           sbc_proto_8_80[ 28 ] * aSamples->m_pTestSamples[ 28 ] +
           sbc_proto_8_80[ 44 ] * aSamples->m_pTestSamples[ 44 ] +
           sbc_proto_8_80[ 60 ] * aSamples->m_pTestSamples[ 60 ] +
           sbc_proto_8_80[ 76 ] * aSamples->m_pTestSamples[ 76 ];

 // 5 mult
 TestY[ 13 ] = sbc_proto_8_80[ 13 ] * aSamples->m_pTestSamples[ 13 ] +
           sbc_proto_8_80[ 29 ] * aSamples->m_pTestSamples[ 29 ] +
           sbc_proto_8_80[ 45 ] * aSamples->m_pTestSamples[ 45 ] +
           sbc_proto_8_80[ 61 ] * aSamples->m_pTestSamples[ 61 ] +
           sbc_proto_8_80[ 77 ] * aSamples->m_pTestSamples[ 77 ];

 // 5 mult
 TestY[ 14 ] = sbc_proto_8_80[ 14 ] * aSamples->m_pTestSamples[ 14 ] +
           sbc_proto_8_80[ 30 ] * aSamples->m_pTestSamples[ 30 ] +
           sbc_proto_8_80[ 46 ] * aSamples->m_pTestSamples[ 46 ] +
           sbc_proto_8_80[ 62 ] * aSamples->m_pTestSamples[ 62 ] +
           sbc_proto_8_80[ 78 ] * aSamples->m_pTestSamples[ 78 ];

  // 5 mult
 TestY[ 15 ] = sbc_proto_8_80[ 15 ] * aSamples->m_pTestSamples[ 15 ] +
           sbc_proto_8_80[ 31 ] * aSamples->m_pTestSamples[ 31 ] +
           sbc_proto_8_80[ 47 ] * aSamples->m_pTestSamples[ 47 ] +
           sbc_proto_8_80[ 63 ] * aSamples->m_pTestSamples[ 63 ] +
           sbc_proto_8_80[ 79 ] * aSamples->m_pTestSamples[ 79 ];

 for( i = 0; i < 16; ++i )
 {
  if( 10 != i )
  {
   if( fabs( TestY[ i ] - mad_f_todouble( Y[ i ] ) * ( 1L << PCM_SHIFT ) ) >= 0.5 )
   {
    fprintf( stderr, "Y[ %d ] = New %lf, Old %lf, New 16 = %lf 64 = %lf 32 = %lf, 48 = %lf, Old 16 = %lf 64 = %lf 32 = %lf, 48 = %lf, SB 0 = %lf Old %lf, SB 16 = %lf Old %lf, SB 32 = %lf Old %lf, SB 48 = %lf Old %lf\n",
          i, mad_f_todouble( Y[ i ] ) * ( 1L << PCM_SHIFT ), TestY[ i ],
          mad_f_todouble( aSamples->m_pSamples[ i  ] ),
          mad_f_todouble( aSamples->m_pSamples[ i + 16 ] ),
          mad_f_todouble( aSamples->m_pSamples[ i + 32 ] ),
          mad_f_todouble( aSamples->m_pSamples[ i + 48 ] ),
          aSamples->m_pTestSamples[ i ],
          aSamples->m_pTestSamples[ i + 16 ],
          aSamples->m_pTestSamples[ i + 32 ],
          aSamples->m_pTestSamples[ i + 48 ],
          mad_f_todouble( sbc_proto_8_80_f[ i ] ),
          sbc_proto_8_80[ i ],
          mad_f_todouble( sbc_proto_8_80_f[ i + 16 ] ),
          sbc_proto_8_80[ i + 16 ],
          mad_f_todouble( sbc_proto_8_80_f[ i + 32 ] ),
          sbc_proto_8_80[ i + 32 ],
          mad_f_todouble( sbc_proto_8_80_f[ i + 48 ] ),
          sbc_proto_8_80[ i + 48 ] );

    assert( fabs( TestY[ i ] - Y[ i ] ) < 0.5 );
   }
  }
 }

 // Initialisation
 for( i = 0; i < 8; ++i )
 {
  tmpTestSB[ i ] = TestY[ 2 ];
 }
 */
/*
  // Reshuffling version - does not work
  // Total 21 multiplications
  // Total for procedure 70 + 21 = 91 multiplications
  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 0 ], ty04 = ( Y[ 0 ] + Y[ 4 ] ) );
  MAD_F_ADD( hi_s[ 0 ], lo_s[ 0 ], hi, lo );
  MAD_F_SUB( hi_s[ 3 ], lo_s[ 3 ], hi, lo );
  MAD_F_SUB( hi_s[ 4 ], lo_s[ 4 ], hi, lo );
  MAD_F_ADD( hi_s[ 7 ], lo_s[ 7 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 0 ] += tmpVal;
  aSubbandSamples[ 3 ] -= tmpVal;
  aSubbandSamples[ 4 ] -= tmpVal;
  aSubbandSamples[ 7 ] += tmpVal;

  tmpDVal = anamatrix8[ 0 ][ 0 ] * ( TestY[ 0 ] + TestY[ 4 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 3 ] -= tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;
  tmpTestSB[ 7 ] += tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 1 ], ty13 = ( Y[ 1 ] + Y[ 3 ] ) );
  MAD_F_ADD( hi_s[ 0 ], lo_s[ 0 ], hi, lo );
  MAD_F_SUB( hi_s[ 7 ], lo_s[ 7 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 0 ] += tmpVal;
  aSubbandSamples[ 7 ] -= tmpVal;

  tmpDVal = anamatrix8[ 0 ][ 1 ] * ( TestY[ 1 ] + TestY[ 3 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 5 ], ty515 = ( Y[ 5 ] - Y[ 15 ] ) );
  MAD_F_ADD( hi_s[ 0 ], lo_s[ 0 ], hi, lo );
  MAD_F_SUB( hi_s[ 7 ], lo_s[ 7 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 0 ] += tmpVal;
  aSubbandSamples[ 7 ] -= tmpVal;

  tmpDVal = anamatrix8[ 0 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 6 ], ty614 = ( Y[ 6 ] - Y[ 14 ] ) );
  MAD_F_ADD( hi_s[ 0 ], lo_s[ 0 ], hi, lo );
  MAD_F_SUB( hi_s[ 1 ], lo_s[ 1 ], hi, lo );
  MAD_F_SUB( hi_s[ 2 ], lo_s[ 2 ], hi, lo );
  MAD_F_ADD( hi_s[ 3 ], lo_s[ 3 ], hi, lo );
  MAD_F_ADD( hi_s[ 4 ], lo_s[ 4 ], hi, lo );
  MAD_F_SUB( hi_s[ 5 ], lo_s[ 5 ], hi, lo );
  MAD_F_SUB( hi_s[ 6 ], lo_s[ 6 ], hi, lo );
  MAD_F_ADD( hi_s[ 7 ], lo_s[ 7 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 0 ] += tmpVal;
  aSubbandSamples[ 1 ] -= tmpVal;
  aSubbandSamples[ 2 ] -= tmpVal;
  aSubbandSamples[ 3 ] += tmpVal;
  aSubbandSamples[ 4 ] += tmpVal;
  aSubbandSamples[ 5 ] -= tmpVal;
  aSubbandSamples[ 6 ] -= tmpVal;
  aSubbandSamples[ 7 ] += tmpVal;

  tmpDVal = anamatrix8[ 0 ][ 6 ] * ( TestY[ 6 ] - TestY[ 14 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 1 ] -= tmpDVal;
  tmpTestSB[ 2 ] -= tmpDVal;
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;
  tmpTestSB[ 7 ] += tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 7 ], ty713 = ( Y[ 7 ] - Y[ 13 ] ) );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 0 ] += tmpVal;
  aSubbandSamples[ 7 ] -= tmpVal;
  MAD_F_ADD( hi_s[ 0 ], lo_s[ 0 ], hi, lo );
  MAD_F_SUB( hi_s[ 7 ], lo_s[ 7 ], hi, lo );

  tmpDVal = anamatrix8[ 0 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 8 ], ty812 = ( Y[ 8 ] - Y[ 12 ] ) );
  MAD_F_ADD( hi_s[ 0 ], lo_s[ 0 ], hi, lo );
  MAD_F_SUB( hi_s[ 3 ], lo_s[ 3 ], hi, lo );
  MAD_F_SUB( hi_s[ 4 ], lo_s[ 4 ], hi, lo );
  MAD_F_ADD( hi_s[ 7 ], lo_s[ 7 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 0 ] += tmpVal;
  aSubbandSamples[ 3 ] -= tmpVal;
  aSubbandSamples[ 4 ] -= tmpVal;
  aSubbandSamples[ 7 ] += tmpVal;

  tmpDVal = anamatrix8[ 0 ][ 8 ] * ( TestY[ 8 ] - TestY[ 12 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 3 ] -= tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;
  tmpTestSB[ 7 ] += tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 9 ], ty911 = ( Y[ 9 ] - Y[ 11 ] ) );
  MAD_F_ADD( hi_s[ 0 ], lo_s[ 0 ], hi, lo );
  MAD_F_SUB( hi_s[ 7 ], lo_s[ 7 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 0 ] += tmpDVal;
  aSubbandSamples[ 7 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 0 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  // S[ 0 ] and S[ 7 ] ready

  MAD_F_ML0( hi, lo, anamatrix8_f[ 1 ][ 0 ], ty04 );
  MAD_F_ADD( hi_s[ 1 ], lo_s[ 1 ], hi, lo );
  MAD_F_SUB( hi_s[ 2 ], lo_s[ 2 ], hi, lo );
  MAD_F_SUB( hi_s[ 5 ], lo_s[ 5 ], hi, lo );
  MAD_F_ADD( hi_s[ 6 ], lo_s[ 6 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 1 ] += tmpVal;
  aSubbandSamples[ 2 ] -= tmpVal;
  aSubbandSamples[ 5 ] -= tmpVal;
  aSubbandSamples[ 6 ] += tmpVal;

  tmpDVal = anamatrix8[ 1 ][ 0 ] * ( TestY[ 0 ] + TestY[ 4 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 2 ] -= tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;
  tmpTestSB[ 6 ] += tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 1 ][ 1 ], ty13 );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 1 ] += tmpVal;
  aSubbandSamples[ 6 ] -= tmpVal;
  MAD_F_ADD( hi_s[ 1 ], lo_s[ 1 ], hi, lo );
  MAD_F_SUB( hi_s[ 6 ], lo_s[ 6 ], hi, lo );

  tmpDVal = anamatrix8[ 1 ][ 1 ] * ( TestY[ 1 ] + TestY[ 3 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 1 ][ 5 ], ty515 );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 1 ] += tmpVal;
  aSubbandSamples[ 6 ] -= tmpVal;
  MAD_F_ADD( hi_s[ 1 ], lo_s[ 1 ], hi, lo );
  MAD_F_SUB( hi_s[ 6 ], lo_s[ 6 ], hi, lo );

  tmpDVal = anamatrix8[ 1 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 1 ][ 7 ],  ty713 );
  MAD_F_ADD( hi_s[ 1 ], lo_s[ 1 ], hi, lo );
  MAD_F_SUB( hi_s[ 6 ], lo_s[ 6 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 1 ] += tmpVal;
  aSubbandSamples[ 6 ] -= tmpVal;

  tmpDVal = anamatrix8[ 1 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 1 ][ 8 ], ty812 );
  MAD_F_ADD( hi_s[ 1 ], lo_s[ 1 ], hi, lo );
  MAD_F_SUB( hi_s[ 2 ], lo_s[ 2 ], hi, lo );
  MAD_F_SUB( hi_s[ 5 ], lo_s[ 5 ], hi, lo );
  MAD_F_ADD( hi_s[ 6 ], lo_s[ 6 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 1 ] += tmpVal;
  aSubbandSamples[ 2 ] -= tmpVal;
  aSubbandSamples[ 5 ] -= tmpVal;
  aSubbandSamples[ 6 ] += tmpVal;

  tmpDVal = anamatrix8[ 1 ][ 8 ] * ( TestY[ 8 ] - TestY[ 12 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 2 ] -= tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;
  tmpTestSB[ 6 ] += tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 1 ][ 9 ], ty911 );
  MAD_F_ADD( hi_s[ 1 ], lo_s[ 1 ], hi, lo );
  MAD_F_SUB( hi_s[ 6 ], lo_s[ 6 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 1 ] += tmpVal;
  aSubbandSamples[ 6 ] -= tmpVal;

  tmpDVal = anamatrix8[ 1 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  // S[1] and S[6] ready now
  
  MAD_F_ML0( hi, lo, anamatrix8_f[ 2 ][ 5 ], ty515 );
  MAD_F_ADD( hi_s[ 2 ], lo_s[ 2 ], hi, lo );
  MAD_F_SUB( hi_s[ 5 ], lo_s[ 5 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 2 ] += tmpVal;
  aSubbandSamples[ 5 ] -= tmpVal;

  tmpDVal = anamatrix8[ 2 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 2 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 2 ][ 7 ], ty713 );
  MAD_F_ADD( hi_s[ 2 ], lo_s[ 2 ], hi, lo );
  MAD_F_SUB( hi_s[ 5 ], lo_s[ 5 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 2 ] += tmpVal;
  aSubbandSamples[ 5 ] -= tmpVal;

  tmpDVal = anamatrix8[ 2 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 2 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 2 ][ 9 ], ty911 );
  MAD_F_ADD( hi_s[ 2 ], lo_s[ 2 ], hi, lo );
  MAD_F_SUB( hi_s[ 5 ], lo_s[ 5 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 2 ] += tmpVal;
  aSubbandSamples[ 5 ] -= tmpVal;

  tmpDVal = anamatrix8[ 2 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 2 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;

  // S[5] and S[5] ready now

  MAD_F_ML0( hi, lo, anamatrix8_f[ 3 ][ 1 ], ty13 );
  MAD_F_ADD( hi_s[ 3 ], lo_s[ 3 ], hi, lo );
  MAD_F_SUB( hi_s[ 4 ], lo_s[ 4 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 3 ] += tmpVal;
  aSubbandSamples[ 4 ] -= tmpVal;

  tmpDVal = anamatrix8[ 3 ][ 1 ] * ( TestY[ 1 ] + TestY[ 3 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 3 ][ 5 ], ty515 );
  MAD_F_ADD( hi_s[ 3 ], lo_s[ 3 ], hi, lo );
  MAD_F_SUB( hi_s[ 4 ], lo_s[ 4 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 3 ] += tmpVal;
  aSubbandSamples[ 4 ] -= tmpVal;

  tmpDVal = anamatrix8[ 3 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 3 ][ 7 ], ty713 );
  MAD_F_ADD( hi_s[ 3 ], lo_s[ 3 ], hi, lo );
  MAD_F_SUB( hi_s[ 4 ], lo_s[ 4 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 3 ] += tmpVal;
  aSubbandSamples[ 4 ] -= tmpVal;

  tmpDVal = anamatrix8[ 3 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;

  MAD_F_ML0( hi, lo, anamatrix8_f[ 3 ][ 9 ], ty911 );
  MAD_F_ADD( hi_s[ 3 ], lo_s[ 3 ], hi, lo );
  MAD_F_SUB( hi_s[ 4 ], lo_s[ 4 ], hi, lo );
  tmpVal = MAD_F_MLZ( hi, lo );
  aSubbandSamples[ 3 ] += tmpVal;
  aSubbandSamples[ 4 ] -= tmpVal;
  
  tmpDVal = anamatrix8[ 3 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;
 
  for( i = 0; i < 8; ++i )
  {
   aSubbandSamples[ i ] = MAD_F_MLZ( hi_s[ i ], lo_s[ i ] );
  }
*/

 // Total 13 * 5 + 2 + 3 = 14 * 5 = 70 multiplications */
/* 
  // Calculate 8 sub-band samples by Matrixing 
  memset( tmpTestSB, 0, 8 * sizeof( double ) );

  // Reshuffling version - floating point - works
  // Total 21 multiplications
  // Total for procedure 70 + 21 = 91 multiplications
  tmpDVal = anamatrix8[ 0 ][ 0 ] * ( TestY[ 0 ] + TestY[ 4 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 3 ] -= tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;
  tmpTestSB[ 7 ] += tmpDVal;

  tmpDVal = anamatrix8[ 0 ][ 1 ] * ( TestY[ 1 ] + TestY[ 3 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 0 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 0 ][ 6 ] * ( TestY[ 6 ] - TestY[ 14 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 1 ] -= tmpDVal;
  tmpTestSB[ 2 ] -= tmpDVal;
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;
  tmpTestSB[ 7 ] += tmpDVal;

  tmpDVal = anamatrix8[ 0 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 0 ][ 8 ] * ( TestY[ 8 ] - TestY[ 12 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 3 ] -= tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;
  tmpTestSB[ 7 ] += tmpDVal;

  tmpDVal = anamatrix8[ 0 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 0 ] += tmpDVal;
  tmpTestSB[ 7 ] -= tmpDVal;

  tmpTestSB[ 0 ] += TestY[ 2 ];
  tmpTestSB[ 7 ] += TestY[ 2 ];
  
  // S[0] and S[7] ready now

  tmpDVal = anamatrix8[ 1 ][ 0 ] * ( TestY[ 0 ] + TestY[ 4 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 2 ] -= tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;
  tmpTestSB[ 6 ] += tmpDVal;

  tmpDVal = anamatrix8[ 1 ][ 1 ] * ( TestY[ 1 ] + TestY[ 3 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 1 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 1 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 1 ][ 8 ] * ( TestY[ 8 ] - TestY[ 12 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 2 ] -= tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;
  tmpTestSB[ 6 ] += tmpDVal;

  tmpDVal = anamatrix8[ 1 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 1 ] += tmpDVal;
  tmpTestSB[ 6 ] -= tmpDVal;

  tmpTestSB[ 1 ] += TestY[ 2 ];
  tmpTestSB[ 6 ] += TestY[ 2 ];  

  // S[1] and S[6] ready now
  
  tmpDVal = anamatrix8[ 2 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 2 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 2 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 2 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 2 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 2 ] += tmpDVal;
  tmpTestSB[ 5 ] -= tmpDVal;

  tmpTestSB[ 2 ] += TestY[ 2 ];
  tmpTestSB[ 5 ] += TestY[ 2 ];  

  // S[2] and S[5] ready now

  tmpDVal = anamatrix8[ 3 ][ 1 ] * ( TestY[ 1 ] + TestY[ 3 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 3 ][ 5 ] * ( TestY[ 5 ] - TestY[ 15 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 3 ][ 7 ] * ( TestY[ 7 ] - TestY[ 13 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;

  tmpDVal = anamatrix8[ 3 ][ 9 ] * ( TestY[ 9 ] - TestY[ 11 ] );
  tmpTestSB[ 3 ] += tmpDVal;
  tmpTestSB[ 4 ] -= tmpDVal;

  tmpTestSB[ 3 ] += TestY[ 2 ];
  tmpTestSB[ 4 ] += TestY[ 2 ];  

  // S[3] and S[4] ready now
  */
  /*
  tmpRound++;
  b= 0;
  for( i = 0; i < 8; ++i )
  {
   if( fabs( mad_f_todouble( aSubbandSamples[ i ] ) * ( 1L << PCM_SHIFT ) - 
       tmpTestSB[ i ] ) > 200 )
   {
    fprintf( stderr, "R %d SB[ %d ] New %lf Old %lf\n", tmpRound, i,
        mad_f_todouble( aSubbandSamples[ i ] ) * ( 1L << PCM_SHIFT ),
        tmpTestSB[ i ] );

    for( j = 0; j < 16; ++j )
    {
     fprintf( stderr, "%d Ana New %lf Old %lf, Y New %lf Old %lf\n",
              j, mad_f_todouble( anamatrix8_f[ i ][ j ] ),
              anamatrix8[ i ][ j ],
              mad_f_todouble( Y[ j ] ) * ( 1L << PCM_SHIFT ),
              TestY[ j ] );
    }

    fprintf( stderr, "\n\n" );
    b = 1;
   }

   if( b )
           exit( 0 );
  }*/
   
  // This works
  // Total 7 * 8 = 56 multiplications
  // Total for procedure 70 + 56 = 126 multiplications  
  MAD_F_ML0( hi, lo, anamatrix8_f[ 0 ][ 0 ], ty04 = ( Y[ 0 ] + Y[ 4 ] ) );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 0 ][ 1 ], ty13 = ( Y[ 1 ] + Y[ 3 ] ) );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 0 ][ 5 ], ty515 = ( Y[ 5 ] - Y[ 15 ] ) );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 0 ][ 6 ], ty614 = ( Y[ 6 ] - Y[ 14 ] ) );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 0 ][ 7 ], ty713 = ( Y[ 7 ] - Y[ 13 ] ) );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 0 ][ 8 ], ty812 = ( Y[ 8 ] - Y[ 12 ] ) );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 0 ][ 9 ], ty911 = ( Y[ 9 ] - Y[ 11 ] ) );
  aSubbandSamples[ 0 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

  MAD_F_ML0( hi, lo, anamatrix8_f[ 1 ][ 0 ], ty04  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 1 ][ 1 ], ty13  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 1 ][ 5 ], ty515 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 1 ][ 6 ], ty614 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 1 ][ 7 ], ty713 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 1 ][ 8 ], ty812 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 1 ][ 9 ], ty911 );
  aSubbandSamples[ 1 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

  MAD_F_ML0( hi, lo, anamatrix8_f[ 2 ][ 0 ], ty04  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 2 ][ 1 ], ty13  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 2 ][ 5 ], ty515 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 2 ][ 6 ], ty614 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 2 ][ 7 ], ty713 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 2 ][ 8 ], ty812 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 2 ][ 9 ], ty911 );
  aSubbandSamples[ 2 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

  MAD_F_ML0( hi, lo, anamatrix8_f[ 3 ][ 0 ], ty04  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 3 ][ 1 ], ty13  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 3 ][ 5 ], ty515 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 3 ][ 6 ], ty614 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 3 ][ 7 ], ty713 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 3 ][ 8 ], ty812 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 3 ][ 9 ], ty911 );
  aSubbandSamples[ 3 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

  MAD_F_ML0( hi, lo, anamatrix8_f[ 4 ][ 0 ], ty04  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 4 ][ 1 ], ty13  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 4 ][ 5 ], ty515 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 4 ][ 6 ], ty614 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 4 ][ 7 ], ty713 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 4 ][ 8 ], ty812 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 4 ][ 9 ], ty911 );
  aSubbandSamples[ 4 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

  MAD_F_ML0( hi, lo, anamatrix8_f[ 5 ][ 0 ], ty04  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 5 ][ 1 ], ty13  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 5 ][ 5 ], ty515 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 5 ][ 6 ], ty614 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 5 ][ 7 ], ty713 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 5 ][ 8 ], ty812 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 5 ][ 9 ], ty911 );
  aSubbandSamples[ 5 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

  MAD_F_ML0( hi, lo, anamatrix8_f[ 6 ][ 0 ], ty04  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 6 ][ 1 ], ty13  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 6 ][ 5 ], ty515 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 6 ][ 6 ], ty614 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 6 ][ 7 ], ty713 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 6 ][ 8 ], ty812 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 6 ][ 9 ], ty911 );
  aSubbandSamples[ 6 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

  MAD_F_ML0( hi, lo, anamatrix8_f[ 7 ][ 0 ], ty04  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 7 ][ 1 ], ty13  );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 7 ][ 5 ], ty515 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 7 ][ 6 ], ty614 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 7 ][ 7 ], ty713 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 7 ][ 8 ], ty812 );
  MAD_F_MLA( hi, lo, anamatrix8_f[ 7 ][ 9 ], ty911 );
  aSubbandSamples[ 7 ] = SBCFixedToPCM( MAD_F_MLZ( hi, lo ) + Y[ 2 ] );

/* 
  // This works
  // Total 7 * 8 = 56 multiplications 
  // Total for procedure 70 + 56 = 126 multiplications
  aSubbandSamples[ 0 ] = 
   anamatrix8[ 0 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 0 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 0 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 0 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 0 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 0 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 0 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );

  aSubbandSamples[ 1 ] = 
   anamatrix8[ 1 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 1 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 1 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 1 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 1 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 1 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 1 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );

  aSubbandSamples[ 2 ] = 
   anamatrix8[ 2 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 2 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 2 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 2 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 2 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 2 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 2 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );

  aSubbandSamples[ 3 ] = 
   anamatrix8[ 3 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 3 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 3 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 3 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 3 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 3 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 3 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );

  aSubbandSamples[ 4 ] = 
   anamatrix8[ 4 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 4 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 4 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 4 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 4 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 4 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 4 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );

  aSubbandSamples[ 5 ] = 
   anamatrix8[ 5 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 5 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 5 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 5 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 5 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 5 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 5 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );

  aSubbandSamples[ 6 ] = 
   anamatrix8[ 6 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 6 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 6 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 6 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 6 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 6 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 6 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );

  aSubbandSamples[ 7 ] = 
   anamatrix8[ 7 ][ 0 ] * ( Y[ 0 ] + Y[ 4 ] ) +
   anamatrix8[ 7 ][ 1 ] * ( Y[ 1 ] + Y[ 3 ] ) +
   Y[ 2 ] +
   anamatrix8[ 7 ][ 5 ] * ( Y[ 5 ] - Y[ 15 ] ) +
   anamatrix8[ 7 ][ 6 ] * ( Y[ 6 ] - Y[ 14 ] ) +
   anamatrix8[ 7 ][ 7 ] * ( Y[ 7 ] - Y[ 13 ] ) +
   anamatrix8[ 7 ][ 8 ] * ( Y[ 8 ] - Y[ 12 ] ) +
   anamatrix8[ 7 ][ 9 ] * ( Y[ 9 ] - Y[ 11 ] );
*/

}

void SBCAnalyze4( SBCSampleList* aSamples, sbc_fixed_t* aSubbandSamples )
{
 assert( NULL != aSamples );
 assert( NULL != aSubbandSamples );

 assert( 0 ); // Not implemented
}
