
/* 
 * File   : UICommand.c
 * Author : Serhiy Tkachenko <Serhiy.Tkachenko@tomtom.com>
 * Desc.  : Implementation of UI command and their retrieval for TomTom medial player
 *      
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *      
 */


#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>

#include <sys/poll.h>
#include <sys/types.h>
#include <sys/stat.h>

#define PROTOCOL_MAJOR 1
#define PROTOCOL_MINOR 0
#define CREATE_TIMEOUT 1000

#include "Debug.h"
#include "UICommand.h"
#include "TTMP.h"

static TTMPServer     server;
static TTMPConnection connection;
static int validConnection=0;


static char* GetSocketName( char* aName )
{
 assert( NULL != aName );

 sprintf( aName, "%s", MEDIAPLAYER_SOCKET_PATH_NAME );

 return aName;
}

int MediaPlayerInitMessageServer()
{
  int  rv = -4;
  char tmp[ 2048 ];

  DBG("Creating server..\n");      
  rv = TTMPServerCreate( &server, CREATE_TIMEOUT, PROTOCOL_MAJOR, PROTOCOL_MINOR );
  
  if( rv != TTMP_ERR_NONE) {
    fprintf( stderr, "TTMP server create error %i\n", rv);
  } else {
    DBG("Set server to listen (%s)..\n",GetSocketName(tmp));      
    rv = TTMPServerListen(server,GetSocketName(tmp));
    if( rv != TTMP_ERR_NONE) {
      fprintf(stderr, "TTMP server listen error %i\n", rv);
    } else {
      DBG("Waiting for client..\n");      

retry_accept:
      rv = TTMPServerAccept(server,&connection);
      if( rv == TTMP_ERR_TIMEOUT) { usleep(100000); goto retry_accept; }
      if( rv != TTMP_ERR_NONE) {
	fprintf(stderr,"TTMP server accept error %i\n", rv);
      } else {
        validConnection=1;
      }
    }
  }

  return rv;
}

// get a new conn if old one died
int MediaPlayerReinitMessageServer()
{
  int rv = -1;

  DBG( "Reinitialising lost connection...\n" );

  if(validConnection) {
    TTMPConnectionDestroy(connection);
    validConnection=0;
  }

  rv = TTMPServerAccept(server,&connection);
  if( rv == TTMP_ERR_NONE) {
    validConnection=1;
  } else if ( rv!=TTMP_ERR_TIMEOUT) {
    fprintf(stderr,"Error in accepting new connection %i\n", rv);
  } 

  return rv;
}

int MediaPlayerDoneMessageServer()
{
  int ec;
  
  ec = TTMPServerDestroy(server);
  if(ec!=TTMP_ERR_NONE) fprintf(stderr,"TTMP server destroy error %i\n",ec);

  if(validConnection) {
    ec=TTMPConnectionDestroy(connection);
    if(ec!=TTMP_ERR_NONE) fprintf(stderr,"TTMP connection destroy error %i\n",ec);
    validConnection=0;
  }

  return 1; 
}

int MediaPlayerGetUICommand( TTMPMessage *msg)
{
  if(validConnection) 
    return TTMPConnectionReceive(connection,msg);
  else
    return TTMP_ERR_CONNEOF;
}

int MediaPlayerSendUIFeedback( TTMPMessage msg )
{
  if(validConnection) 
    return TTMPConnectionSend(connection,msg);
  else 
    return TTMP_ERR_CONNEOF;
}

// Client messages
int MediaPlayerInitMessageClient()
{
  int rv = -1;
  char tmp[2048];
  
  rv = TTMPConnectionCreate( &connection, CREATE_TIMEOUT, PROTOCOL_MAJOR, PROTOCOL_MINOR );
  if( rv != TTMP_ERR_NONE ) {
    fprintf(stderr,"TTMP connection create error %i\n", rv);
  } else {
    rv = TTMPConnectionConnect(connection,GetSocketName(tmp));
  }

  return rv;
}

int MediaPlayerDoneMessageClient()
{
 int ec;

 ec=TTMPConnectionDestroy(connection);
 if(ec!=TTMP_ERR_NONE) fprintf(stderr,"TTMP destroy connection error %i\n",ec);

 return 1;
}

int MediaPlayerSendUICommand( TTMPMessage msg )
{
 return TTMPConnectionSend(connection,msg);
}

int MediaPlayerGetUIFeedback( TTMPMessage *msg )
{
  return TTMPConnectionReceiveBlocking(connection,msg);
}
