/* 
 * File   : UICommand.h
 * Author : Serhiy Tkachenko <Serhiy.Tkachenko@tomtom.com>
 * Desc.  : Definition of UI command and their retrieval for TomTom medial player
 *      
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *      
 */


#ifndef __UI_COMMAND_H__
#define __UI_COMMAND_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "TTMP.h"

// Used by application to start a demo movie
#define DEMO_MOVIE_COMMAND_LINE "/mnt/sdcard/bin/demoplayer.sh %s"

#ifdef DEBUG_PATH
#define MP3_PLAYER_PATH "/mnt/sdcard/bin/mp3d"
#else
#define MP3_PLAYER_PATH "/mnt/sdcard/bin/mp3d -n -2"
#endif
#define A2DP_CMD "/bin/a2dpd -n -4"

#ifdef _WIN32

#define AUDIOBOOK_LIST_FILE_NAME  "./audiobook_list.txt"
#define AUDIBLE_CODEC_LIB_NAME    "audible.dll"
#define AUDIOBOOK_IMAGE_FILE_NAME "./audiobook_image.jpg"
#define AUDIOBOOKS_DIRECTORY      "./Audiobooks"   
#define AUDIOBOOKS_SCAN_APP       "AAScan.exe"    
#else

#define AUDIOBOOK_LIST_FILE_NAME  "/var/run/audiobook_list.dat"
#define AUDIBLE_CODEC_LIB_NAME    "/lib/audible.so"
#define AUDIOBOOK_IMAGE_FILE_NAME "/var/run/audiobook_image.jpg"
#define AUDIOBOOKS_DIRECTORY      "/mnt/sdcard/Audiobooks"   
#define AUDIOBOOKS_SCAN_APP       "/bin/AAScan"    

#endif

#define MEDIAPLAYER_SOCKET_PATH_NAME    "/var/run/mediaplayer.cmd"


#define AUDIOBOOK_FILE_EXT              "aa"

#define AA_LIST_KEYWORD_CODEC           "CODEC"
#define AA_LIST_KEYWORD_FILE            "FILE"
#define AA_LIST_KEYWORD_TITLE           "TITLE"
#define AA_LIST_KEYWORD_AUTHOR          "AUTHOR"
#define AA_LIST_KEYWORD_DESCRIPT        "DESCRIPTION"
#define AA_LIST_KEYWORD_CHAPTERS        "CHAPTERS"
#define AA_LIST_KEYWORD_OFFSETS         "OFFSETS"
#define AA_LIST_KEYWORD_TOTALTIME       "TOTALTIME"


// Command codes
typedef enum { 
 EStartPlayback = 0x01,
 EPausePlayback = 0x02,
 EStopPlayback  = 0x03,
 ESkip          = 0x04,
 EFeedback      = 0x07,
 EFeatureMode   = 0x08,

 // Audiobooks support
 ESetChapter       = 0x20,
 ENextChapter      = 0x21,
 EPrevChapter      = 0x22,
 ESetPlaybackSpeed = 0x23 
} ui_command_code_t;

//  Fast forward/reverse modes
typedef enum
{
 EStop        = 0x00,
 EFastForward = 0x01,
 EFastReverse = 0x02
} ui_playback_feature_mode_t;

// Stop modes
typedef enum
{
 EStopNormal      = 0x00,
 EStopSuddenDeath = 0x01
} ui_stop_mode;

// Pause modes
typedef enum
{
 EPauseOff        = 0,
 EPauseOn         = 1
} ui_pause_mode;

// Feedback codes
typedef enum 
{
  EGetPlayPos     = 0x80,
  EGetCurrentSong = 0x81,
  EGetStatus      = 0x82
} ui_feedback_code_t;

typedef enum { ELow = 40, EMedium = 56, EHigh = 64 } bluetooth_quality_t;

// TTMP message field names
#define MEDIAPLAYER_MESSAGE_CODE_FIELD_NAME        "code"
#define MEDIAPLAYER_MESSAGE_TIMESTAMP_FIELD_NAME   "timestamp"
#define MEDIAPLAYER_MESSAGE_PERCENTAGE_FIELD_NAME  "percentage"
#define MEDIAPLAYER_MESSAGE_CURRENTFILE_FIELD_NAME "currentfile"
#define MEDIAPLAYER_MESSAGE_NEXTFILE_FIELD_NAME    "nextfile"
#define MEDIAPLAYER_MESSAGE_ERRORTEXT_FIELD_NAME   "errortext"
#define MEDIAPLAYER_MESSAGE_RESULT_FIELD_NAME      "result"
#define MEDIAPLAYER_MESSAGE_FILENAME_FIELD_NAME    "filename"
#define MEDIAPLAYER_MESSAGE_CODECNAME_FIELD_NAME   "codecname"
#define MEDIAPLAYER_MESSAGE_FEEDBACK_FIELD_NAME    "feedback"
#define MEDIAPLAYER_MESSAGE_INDEX_FIELD_NAME       "index"
#define MEDIAPLAYER_MESSAGE_SPEED_FIELD_NAME       "speed"
#define MEDIAPLAYER_MESSAGE_MODE_FIELD_NAME        "mode"
#define MEDIAPLAYER_MESSAGE_OFFSET_FIELD_NAME      "offset"

// Server messages
int MediaPlayerInitMessageServer();
int MediaPlayerReinitMessageServer();
int MediaPlayerDoneMessageServer();
int MediaPlayerGetUICommand( TTMPMessage *msg );
int MediaPlayerSendUIFeedback( TTMPMessage msg );

// Client messages
int MediaPlayerInitMessageClient();
int MediaPlayerDoneMessageClient();
int MediaPlayerSendUICommand( TTMPMessage command );
int MediaPlayerGetUIFeedback( TTMPMessage *feedback );

#ifdef __cplusplus
}
#endif

#endif // __UI_COMMAND_H__
