/* 
 * File   : UIController.h
 * Author : Serhiy Tkachenko <Serhiy.Tkachenko@tomtom.com>
 * Desc.  : UI controller for TomTom sound player
 *      
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *      
 */


#ifndef __UI_CONTROLLER_H__
#define __UI_CONTROLLER_H__

#ifdef __cplusplus
extern "C" {
#endif

typedef struct 
{
 // Playback control 
 int ( *iStartPlayback )( const char*  aFileName, const char* aCodecName, unsigned long aByteOffset );
 int ( *iPausePlayback )( int aMode );
 int ( *iStopPlayback )( int aMode );
 int ( *iSkip )( const char* aFileName );
 
 // Status getters
 int           ( *iGetCurrentPlayPercentage )( void );
 unsigned long ( *iGetCurrentPlaybackPosition )( void ); 
 unsigned long ( *iGetCurrentChapter )( void ); 
 unsigned long ( *iGetCurrentByteOffset )( void ); 

 const char* ( *iGetCurrentSongName )( void ); 
 const char* ( *iGetNextSongName )( void );
 const char* ( *iGetLastError )( void );

 // Advanced control 
 int ( *iFeatureMode )( int aMode, int aSpeed );
 int ( *iGoToChapter )( int aChapterNumber, int aAbsolute );
} ui_contoller_t;   
 

void InitUIController( ui_contoller_t* aController );
void DoneUIController( ui_contoller_t* aController );

typedef enum { EExitIfError, ENoExitIfError } EErrorBehaviour;

// Resturns 0 in case of success
int  DispatchNextUIMessage( ui_contoller_t* aController, EErrorBehaviour aExitMode );

#ifdef __cplusplus
}
#endif

#endif // __UI_CONTROLLER_H__

