
#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <dlfcn.h>
#include <assert.h>
#include <string.h>


#include "Debug.h"
#include "ReadFile.h"


static void  DestroyPlugInFileProducer( struct __fileReaderData *fileReader )
{
 DBG( "Destroyng PlugIn...\n" );

 assert( NULL != fileReader );
 assert( NULL != fileReader->internalData );

 file_reader_plugin_internal_data_t* tmpIData = ( file_reader_plugin_internal_data_t* )fileReader->internalData;
 
 DBG( "Destroyng new ...\n" );
 tmpIData->iDestroyNewFileProducer( fileReader );
 
 dlclose( tmpIData->iDLLHandle );

 DBG( "Destroyng old ...\n" );
 tmpIData->iDestroyOldFileProducer( fileReader );

 DBG( "Destroyng done\n" );
}


static void* GetDLLSymbol( void* aDLL, const char* aSymbolName, void* aPrevValue )
{
 assert( NULL != aDLL );
 assert( NULL != aSymbolName );

 void* tmpAddr = dlsym( aDLL, aSymbolName );
 char* sbc_dl_err;
 
 if( ( sbc_dl_err = dlerror() ) != NULL )  {
    DBG( "Unable to load symbol %s: %s, setting to prev value %p\n", aSymbolName, sbc_dl_err, aPrevValue );
    tmpAddr = aPrevValue;
  }

 return tmpAddr;
}


fileReaderData* InitPlugInFileProducer( const char * aFileName, const char* aPlugInName, unsigned long aByteOffset, char* aErrorText )
{
 DBG( "Loading file reader PlugIn %s, file to play %s...\n", aPlugInName, aFileName );

 assert( NULL != aFileName );
 assert( NULL != aPlugInName );
 assert( NULL != aErrorText );

 fileReaderData* tmpResult = InitMP3FileProducer( aFileName, aByteOffset );

#ifdef DEBUG
 if( aByteOffset )
 {
  DBG( "Byte offset %ld ignored\n", aByteOffset );
 }
#endif

 *aErrorText = 0;

 if( NULL != tmpResult )
 {
  file_reader_plugin_internal_data_t* tmpIData = malloc( sizeof( file_reader_plugin_internal_data_t ) );

  tmpResult->internalData = tmpIData;  
  
  if( NULL == tmpResult->internalData )
  {
   strcpy( aErrorText, "Can not allocate memory for internal data" );

   DBG( "%s\n", aErrorText );

   goto free_and_exit;
  }

  // Saving old pointers
  tmpIData->iDestroyOldFileProducer = tmpResult->iDestroyFileProducer;
  tmpIData->iOldGetStreamParameters = tmpResult->iGetStreamParameters;

  // Loading DLL
  DBG( "Loading dynamic object \"%s\"\n", aPlugInName );
  
  tmpIData->iDLLHandle = dlopen( aPlugInName, RTLD_NOW );

  if( !tmpIData->iDLLHandle ) 
  {
    sprintf( aErrorText, "Audio input plugin library %s not loaded: %s", aPlugInName, dlerror() ); 

    DBG( "%s\n", aErrorText );

    goto free_and_exit;
  }

  tmpResult->iSetFileProducerNextFile  = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInSetNextFile", tmpResult->iSetFileProducerNextFile );
  tmpResult->iFileProducer             = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInFileProducer", tmpResult->iFileProducer );
  tmpResult->iDecoderProducer          = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInDecoderProducer", tmpResult->iDecoderProducer );
  tmpIData->iDestroyNewFileProducer    = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInDestroyFileProducer", NULL );
  tmpResult->iInitReadFile             = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInInitReadFile", tmpResult->iInitReadFile );
  tmpResult->iCloseReadFile            = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInCloseReadFile", tmpResult->iCloseReadFile );
  tmpResult->iReadFileData             = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInReadFileData", tmpResult->iReadFileData );
  tmpResult->iEOF                      = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInEOF", tmpResult->iEOF );
  tmpResult->iStartFeatureMode         = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInStartFeatureMode", tmpResult->iStartFeatureMode );
  tmpResult->iGoToChapter              = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInGoToChapter", tmpResult->iGoToChapter );
  tmpResult->iGetStreamParameters      = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInGetStreamParameters", NULL );
  tmpResult->iDecodeFrame              = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInDecodeFrame", tmpResult->iDecodeFrame );
  tmpResult->iBytePosToChapter         = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInBytePosToChapter", tmpResult->iBytePosToChapter );

  if( NULL == tmpResult->iSetFileProducerNextFile ||
      NULL == tmpResult->iFileProducer            ||
      NULL == tmpResult->iDecoderProducer         ||
      NULL == tmpIData->iDestroyNewFileProducer   ||   
      NULL == tmpResult->iInitReadFile            ||
      NULL == tmpResult->iCloseReadFile           ||
      NULL == tmpResult->iReadFileData            ||
      NULL == tmpResult->iEOF                     ||
      NULL == tmpResult->iGoToChapter             ||
      NULL == tmpResult->iStartFeatureMode        ||
      NULL == tmpResult->iGetStreamParameters     ||
      NULL == tmpResult->iDecodeFrame             ||
      NULL == tmpResult->iBytePosToChapter )
  {
   dlclose( tmpIData->iDLLHandle );
   free( tmpIData );

   strcpy( aErrorText, "Not all PlugIn function defined" );

   DBG( "%s\n", aErrorText );

   goto free_and_exit;
  }

  tmpResult->iDestroyFileProducer = DestroyPlugInFileProducer; 

  // Initialising PlugIn
  InitPlugInFunc tmpInitFunc = GetDLLSymbol( tmpIData->iDLLHandle, "PlugInInit", NULL );

  if( NULL != tmpInitFunc )
  {
   tmpInitFunc();
  }

  // Initializing file read
  if( tmpResult->iInitReadFile( tmpResult, tmpResult->fp, tmpResult->curr_file_name, aErrorText ) )
  {
   DBG( "Can not initisalise PlugIn file reader: %s !\n", aErrorText );

   goto free_and_exit;
  }
 }
 else
 {
  strcpy( aErrorText, "Can not initialise basic file producer" );
  
  DBG( "%s\n", aErrorText );
 }

 return tmpResult;

free_and_exit:
 tmpResult->iDestroyFileProducer( tmpResult );

 return NULL;
}
