/* 
 * File   : TomTomProjects/AudioStreaming/StreamBuffer.h
 * Author : Koen Martens <koen.martens@tomtom.com>
 * Desc.  : API for mp3 decoder with synchronised output
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __STREAM_BUFFER_H__
#define __STREAM_BUFFER_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <pthread.h>
   
// a buffer handle
typedef struct {
  pthread_t         thread;
  pthread_attr_t    thread_attr;
  pthread_mutex_t   flipMutex;
  pthread_cond_t    flipCond;
  pthread_mutex_t   flushMutex;
  pthread_cond_t    flushCond;
  unsigned char    *buffer;
  int               bufferSize;
  int               bufferFreeSize;
  unsigned char    *bufferStart[2];
  volatile size_t   available[2];
  volatile size_t   offset[2];
  volatile int      eos[2];
  volatile int      currBuff;
  volatile int      flip;
  volatile int      flush;
  volatile int      killThread;
  volatile int      interruptFlush;
  pthread_mutex_t   flippedMutex;	// lock for flipped
  volatile int      flipped;	// set by ReadStreamBuffer, reset by FlipStreamBuffer
  void *            udata;
  int		        debug;	// remove!
  int               external_buffer;

  int               fake; // This is just a pass-through buffer
  int ( *FakeReadFunction )( void**ptr, int len );  
} StreamBufferHandle;


#define BO_IBLK           (1<<0)	// block init until data available
#define BO_NOIBLK         (0<<0)   // don't block init
#define BO_IFAKE          (1<<1)   // Fake pass-throug buffer
#define BO_EXTERNALBUFFER (1<<2)   // Buffer is allocated/deallocated externally

inline int GetStreamBufferIndex(StreamBufferHandle *h);

inline unsigned char *GetStreamBufferPtr(StreamBufferHandle *h);

inline unsigned char *GetStreamBufferEnd(StreamBufferHandle *h);

inline int GetStreamBufferLength(StreamBufferHandle *h);

inline int GetStreamBufferAvailable(StreamBufferHandle *h);

inline void *GetStreamBufferUdata(StreamBufferHandle *h);

inline int StreamBufferKillThread(StreamBufferHandle *h);

inline void StreamBufferEos(StreamBufferHandle *h);

inline int  StreamBufferIsEos( StreamBufferHandle *h );

int FlushStreamBuffer(StreamBufferHandle *h, int size);

int FlushStreamBufferAsync(StreamBufferHandle *h, int size);

void UnflushStreamBuffer(StreamBufferHandle *h);

StreamBufferHandle *InitStreamBuffer( size_t bufferSize, void *(*producer)(void *), void *arg, int options);

StreamBufferHandle *InitStreamBufferSet( size_t bufferSize, void *(*producer)(void *),
                                                void *arg, int options, void *buffer);

int ReadStreamBuffer(StreamBufferHandle *h,void* *ptr,int len);
int FlippedStreamBuffer(StreamBufferHandle *h);
int WriteStreamBuffer( StreamBufferHandle *h,void* ptr,int len );

void DiscardStreamBuffer( StreamBufferHandle *h );
void DiscardStreamBufferFull( StreamBufferHandle *h );

int StreamBufferWaitFlush(StreamBufferHandle *h);

void InterruptFlushStreamBuffer(StreamBufferHandle *h);

void StreamBufferResetEos(StreamBufferHandle *h);

void DoneStreamBufferWakeUp(StreamBufferHandle *h, int wakeupAudioAccept, int wakeUps, ...);
void DoneStreamBuffer(StreamBufferHandle *h);


#ifdef __cplusplus
}
#endif

#endif // __STREAM_BUFFER_H__

/* EOF */
