/* 
 * File   : A2dp.c
 * Author : Koen Martens <koen.martens@tomtom.com>
 * Desc.  : A2dp protocol implementation, data structures
 *          and function prototypes
 *      
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *      
 */


#ifndef __A2DP_H
#define __A2DP_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/socket.h>
    
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

#include "A2dpTypes.h"
#include "A2DPListen.h"

#define NONSPECAUDIO    1

#define MONO            0
#define DUAL_CHANNEL    1
#define STEREO          2
#define JOINT           3

#define LOUDNESS        0
#define SNR             1

#define MIN_BITPOOL      10
#define MAX_BITPOOL      128

#define MIN_LINK_QUALITY  200
#define LINK_CHECK_PERIOD 50
#define LINK_BAD_TRESHOLD 3
    
#define DEFAULT_HCI_DEVICE 0
    
// This is used only for debugging, should be set to 0 normally
#define DEFAULT_SIGNALLING_MTU 0 //48

typedef struct {
  int                 device_ctl;
  struct hci_dev_info device_info;
  // 
  unsigned short   psm_cmd;
  unsigned short   psm_stream;
  int              cmdfd;			            // command socket
  int              streamfd;			        // stream socket
  uint16_t         mtu;
  uint16_t         signalling_mtu;              // mtu for signalling channel
  bdaddr_t         src;
  bdaddr_t         dst;
  unsigned long    flags;			            // workaround flags
  uint16_t         vendor, product, version;	// headphone info
  int              transaction;			        // transaction ID, incrementing
  int              seid;
  int              paused;
  int              link_check;
  int              link_bad;
  int              started;                     // streaming is started
  int              configured;                  // SEP is configured
  //
  int              nrframes;			        // nr of frames to fit into the mtu
  uint8_t          reassembled_packet[ MAX_PACKET_SIZE ];
  int              reassembled_packet_size;
  
  // headers
  struct media_packet_header  packet_header;
  struct media_payload_header payload_header;

  time_t timestamp;
  uint16_t seq_num;

  // Listeneing stuff
  a2dp_listen_t   listen_data;
} a2dp_packager_t;


a2dp_packager_t *InitA2dp( packager_t *p, const char *dstaddr, char* aErrorText );
int              ConnectA2dp( void* packager, const char *dstaddr, char* aErrorText ); 
int              CreateA2dpPackage( void *packager );
void             DoneA2dp( void *packager );
int              PauseA2dp( void *packager, char* aErrorText );
int              ResumeA2dp( void *packager, char* aErrorText );
int              ProcessA2DPSinkRequests( void *packager );

//void PauseA2dp(void *p,int pause);

#ifdef __cplusplus
}
#endif

#endif //  __DECODER_H__

/* EOF */
