#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>
#include <errno.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <limits.h>
#include <sched.h>

#include "Debug.h"
#include "StreamBuffer.h"
#include "Packager.h"
#include "A2dp.h"
#include "A2dpPlayer.h"

#ifndef A2DP_DEBUG_FILE
#undef DEBUG
#endif

unsigned long shouldHaveSent(struct timeval old,struct timeval new,unsigned long duration) {
  unsigned long seconds,tusecs,usecs;
  unsigned long rv;
  unsigned long realusec;
 
  if(old.tv_sec==new.tv_sec) {
    seconds=0; usecs=(new.tv_usec-old.tv_usec);
  } else {
    seconds=new.tv_sec-old.tv_sec-1;
    tusecs=((1000000-old.tv_usec) + new.tv_usec);
    usecs=tusecs%1000000; seconds+=tusecs/1000000; 
  }
//  DBG("secs,usecs = %lu,%lu\n",seconds,usecs);

  realusec=((seconds*1000000)+usecs);

  
  rv=realusec/duration;

	// fudge factor, corrects for expected timer inaccuracy
//  if( (realusec + 20000)/duration > rv ) rv++;

//  DBG("rv=%lu\n",rv);

  return rv;
}

void *a2dpPlayer(void *arg);

a2dp_player_t *InitA2dpPlayer(packager_t *p, char* aErrorText ) {
  a2dp_player_t *h;

  DBG("Creating a2dpPlayer..\n");

  assert( NULL != aErrorText );
  *aErrorText = 0;

  h = malloc( sizeof( a2dp_player_t ) );
  if( NULL == h ) 
  {
      strcpy( aErrorText, "Can not allocate memory for A2DP player\n" );
      fprintf( stderr, "%s\n", aErrorText );

      return NULL; 
  }

  memset( h, 0, sizeof( a2dp_player_t ) );

  pthread_mutex_init(&(h->killMutex),0);
  pthread_cond_init(&(h->killCond),0);

  h->currentPos = 0;
  h->packager   = p;

  if(pthread_create(&(h->thr),0,a2dpPlayer,h)) {
    strcpy( aErrorText, "Can not create A2DP player thread\n" );
    fprintf( stderr, "%s\n", aErrorText );
    
    free(h);
    return NULL;
  }

  return h;
}

void DestroyA2dpPlayer(a2dp_player_t *h) {

  DBG( "Destroying A2DP player...\n" );

  pthread_mutex_lock(&(h->killMutex));
  h->killThread = 1;
  pthread_cond_broadcast(&(h->killCond));
  pthread_mutex_unlock(&(h->killMutex));

  pthread_join(h->thr,NULL);

  free(h);

  DBG( "Done\n" );
}

/*
static void WaitForBufferFullness( packager_t* p )
{
  int tmpCount = 50;

  DBG( "Waiting to A2DP buffer to fill...\n");

  while( GetStreamBufferAvailable( p->out ) < GetStreamBufferLength( p->out ) * 9 / 10 )
  {
   DBG( "A2DP buffer not full: avail %d, len %d, try %d\n", GetStreamBufferAvailable( p->out ), GetStreamBufferLength( p->out ), tmpCount );
   
   usleep( 100000 );

   if( StreamBufferIsEos( p->out ) )
   {
    DBG( "A2DP buffer EOS!!!\n" );
    break;
   }
   else
    if( !tmpCount-- )
    { 
     DBG( "FATAL: A2DP buffer never full!!!\n" );
     break;
    }   
  }
}
*/

void *a2dpPlayer(void *arg) {
  a2dp_player_t    *h;
  packager_t       *p;
  a2dp_packager_t  *a;


#ifdef DEBUG
  int prevtime = 0, currtime;
#endif

#ifdef A2DP_DEBUG_FILE
FILE* a2dp_debug_file = fopen( "/mnt/sdcard/avdtp.media", "w+b" );
int   a2dp_pack_count = 20;
#endif

  // 
  int thisBurst;
  int done = 0;
  int backlogExceeded=0;
  //
  struct timeval startTime, currTime, lastTime;
#ifdef DEBUG
  struct timeval nowTime, lastPTime, nowPTime;
  unsigned long  lastSent     = 0;
#endif
  int            writeError   = 0;
  int            packetPeriod = 0;
  double         doublePeriod = 0.0;
  unsigned long  haveSent     = 0, 
		 leap         = 0,
		 leapofs      = 0,
                 shouldSend   = 0,
                 missed       = 0,
		 leapPackets  = 0;
  //
#ifdef DEBUG
  unsigned long maxPPeriod=0,minPPeriod=-1,totalPPeriod=0,countPPeriod=0;
  unsigned long burstTotal = 0,
                burstCount = 0,
                burstMax   = 0,
                burstMin   = -1;
#endif



  DBG("entering thread..\n");

  assert(arg!=NULL);

  h=(a2dp_player_t*)arg;

  assert(h->packager!=NULL);

  p=h->packager;
 
  assert(p->udata!=NULL);

  a=(a2dp_packager_t *)(p->udata);

  packetPeriod = p->encoder->sbc.duration * a->nrframes;

	// calculate remainder and leap packet frequency
  doublePeriod = p->encoder->sbc.durationDouble * ((double)(a->nrframes));
  leap = doublePeriod/(doublePeriod-((double)packetPeriod));
  leapofs = (leap>3000)?(leap-3000):0;
  

  DBG("------========> doublePeriod %.12f\n",doublePeriod);
  DBG("------========> packetPeriod %.12f\n",(double)packetPeriod);
  DBG("------========> remainder    %.12f\n",(doublePeriod-((double)packetPeriod)));
  DBG("------========> calculated leap: %lu \n",leap);

  //WaitForBufferFullness( p );

  // scheduler experiment
  {
    static struct sched_param sp;
    static struct sched_param spb;
    sp.sched_priority=1;
    if(sched_setscheduler(0,SCHED_FIFO,&sp)) {
      DBG("!!!!!!!!!!!!!! unable to set scheduler policy\n");
      pthread_exit((void*)-1);
    }

    if(sched_getscheduler(0)!=SCHED_FIFO) {
      DBG("!!!!!!!!!!!!!! scheduler policy not FIFO\n");
      pthread_exit((void*)-1);
    }
   
    if(sched_getparam(0,&spb)) {
      DBG("!!!!!!!!!!!!!! can't get schedule param\n");
      pthread_exit((void*)-1);
    }
    DBG("!!!!!!!!!!!!! schedule param: %i\n",spb.sched_priority);

        // renice ourselves to not be nice (we need accurate timing),
        // do this _before_ forking off any threads
//    errno=0;
//    if( (nice(-15)==-1) && (errno!=0) ) {
//      perror(__func__);
//      exit(1);
//    }
 
  }
  // scheduler experiment


  gettimeofday( &lastTime, NULL );
  if( gettimeofday( &startTime, NULL ) != 0 )
  {
   DBG( "FATAL: Can not gettimeofday(), exiting !!!\n" );
   goto exit_play;
  }

	// main loop
  DBG("starting A2DP loop..\n");

  while( !done && !writeError && !h->killThread ) {    
//    struct timespec req;


#ifdef DEBUG
    if( !h->paused )
    {
      currtime=time(0);
     if(currtime>prevtime+5) { 
      if( haveSent )
      {
        unsigned long period;

	if(lastTime.tv_sec==nowTime.tv_sec) {
	  period=nowTime.tv_usec-lastTime.tv_usec;
	} else {
	  unsigned long perS=nowTime.tv_sec-lastTime.tv_sec-1;
	  unsigned long perU=(1000000-lastTime.tv_usec)+nowTime.tv_usec;
	  period=(perS*1000000)+perU;
	}

        DBG("packets=%lu, missed=%lu (%.2f), leap=%lu, avgdur=%f, should sent=%lu\n",
			haveSent,
			missed,
			((float)(missed)*100.0)/(float)haveSent,
			leapPackets,
			((double)period)/((double)(haveSent-lastSent)),
            shouldSend
			);
	DBG("maxPPeriod=%lu, minPPeriod=%lu, avgPPeriod=%f\n",
		maxPPeriod,
		minPPeriod,
		((double)totalPPeriod)/((double)countPPeriod));

	maxPPeriod=0; minPPeriod=-1; totalPPeriod=0; countPPeriod=0;

	lastSent=haveSent; lastTime=nowTime;
      }

      if(burstCount)
        DBG("burstAvg=%lu, burstMax=%lu, burstMin=%lu\n",burstTotal/burstCount,burstMax,burstMin);
      burstMax=0; burstMin=-1;
      prevtime=currtime;
     }
    }
#endif

//    gettimeofday( &currTime, NULL ); 
//    if ( ( currTime.tv_sec - startTime.tv_sec ) > 4200 ) {
//	startTime=currTime; haveSent=0; leapPackets=0;
//    }
//    shouldSend  = shouldHaveSent( startTime, currTime, packetPeriod )+leapPackets;
	// sleep for packet duration, however the resolution is not
        // assumed to be sufficient
//    req.tv_sec=0; req.tv_nsec=packetPeriod*1000;
//    req.tv_sec=0; req.tv_nsec=1; // packetPeriod*1000;
//    while( nanosleep(&req,&req)==-1 ) {
//    while( (shouldSend-haveSent) && nanosleep(&req,&req)==-1 ) {
//      if(errno!=EINTR) { DBG("nanosleep error\n"); fflush(stderr); break; }
//#ifdef DEBUG
//      if(errno==EINTR) { DBG("nanosleep interrupted\n"); fflush(stderr); break; }
//#endif      
//    }
    usleep(1);
        // 

    // Pause check
    if( h->paused )
    {
       // Resetting counters
      gettimeofday( &startTime, NULL );
      missed = shouldSend = haveSent = 0; leapPackets=0;

#ifdef DEBUG
      static int tmpCount = 0;

      if( tmpCount++ > 200 )
      {
       DBG( "A2DP Player paused, buffer fill %d\n", GetStreamBufferAvailable( p->out ) );
       tmpCount = 0;
      }
#endif 
	// sleep 10ms, then recheck, better would be a pthread_cond_wait of course
      usleep(10000);
      continue;
    }
    
    if( h->resumed )
     {
      h->resumed = 0;

      DBG( "A2DP Player resumed, buffer fill %d\n", GetStreamBufferAvailable( p->out ) );

      //WaitForBufferFullness( p );

      gettimeofday( &startTime, NULL );
      missed = shouldSend = haveSent = 0; leapPackets=0;
     }


	// FIXME: 4200 is hardcoded (approx) result of 2^32 / 1000000, which is the number
	// of seconds that fits in an unsigned long when expressed as useconds, we want
	// to reset the starttime when that happens, because shouldHaveSent doesn't handle
	// overflows
    gettimeofday( &currTime, NULL ); 
    if ( ( currTime.tv_sec - startTime.tv_sec ) > 4200 ) {
	startTime=currTime; haveSent=0; leapPackets=0;
    }
    shouldSend  = shouldHaveSent( startTime, currTime, packetPeriod )+leapPackets;

#ifdef DEBUG
    burstCount++;
    burstTotal+=(shouldSend)-haveSent;
    if(burstMax<(shouldSend-haveSent)) burstMax=shouldSend-haveSent;
    if(burstMin>((unsigned long)(shouldSend-haveSent))) burstMin=(shouldSend)-haveSent;
#endif

	// if difference too big for too long, forget it will never catch up

//    DBG("shouldSend, haveSent = %i, %i\n",shouldSend,haveSent);
    if ( /*0 &&*/  (shouldSend-haveSent) > 6 ) {
	   backlogExceeded++;
//    } 
     if(backlogExceeded>20)
     {
      DBG( "Restaring A2DP due to numerous errors\n" );
      goto exit_play; // Restart all this
     }
     else {
//    if(backlogExceeded>20) {
	startTime=currTime; haveSent=0; leapPackets=0;
	backlogExceeded=0;
	DBG("backlog limit, reset\n");
    continue;}
    } 

	// send at most 2 packets right after each other to prevent 
	// flooding certain el-cheapo headphones
    for(thisBurst=2;(haveSent<shouldSend) && thisBurst;thisBurst--) {
      int ec;
      void *ptr;
    
	// Leap packet adjustment
        // ((haveSent-leapofs)<haveSent) actually checks whether haveSent-leapOfs>0, 
	// but since haveSent & leapOfs are unsigned, it is always >0..
      if( ((haveSent-leapofs)<haveSent) && !((haveSent-leapofs) % leap)) {
	DBG("leap!\n");
//	leapPackets++;
//	haveSent++;
      }

      //DBG( "ReadStreamBuffer()\n" );
      ec = ReadStreamBuffer( p->out, 
                               &ptr,
                               p->packetlen );
        

      if( ec == 0 ) {		// no packets available
        if( h->sync )
        {
         DBG( "Buffer synced, pausing...\n" );
         h->sync   = 0;
         h->paused = 1;
         //a->paused = 1;
         break;
        }
        else
        {        
	     missed++;

         DBG( "No media packet read !\n" );  

         usleep(packetPeriod);	// wait and try again
	 gettimeofday(&startTime,0); 
         shouldSend = haveSent = 0;
	     break;
        }
      } else if( ec == -1 ) {	// end of stream

        DBG("No more packets!\n");

        done = 1;
        break;
      } else if( ec!=p->packetlen ) {

        DBG("Incomplete packet read (%i, %d)!\n",ec, p->packetlen);

	missed++;
        usleep(packetPeriod);	// wait and try again
      } else {

#ifdef DEBUG
	{
	  gettimeofday(&nowPTime,NULL);
          unsigned long period;

	  if(lastPTime.tv_sec==nowPTime.tv_sec) {
	    period=nowPTime.tv_usec-lastPTime.tv_usec;
	  } else {
	    unsigned long perS=nowPTime.tv_sec-lastPTime.tv_sec-1;
	    unsigned long perU=(1000000-lastPTime.tv_usec)+nowPTime.tv_usec;
	    period=(perS*1000000)+perU;
	  }

	  if(maxPPeriod<period) maxPPeriod=period;
	  if(minPPeriod>period) minPPeriod=period;
	  totalPPeriod+=period; countPPeriod++;
	  lastPTime=nowPTime;
	}
#endif

#ifdef A2DP_DEBUG_FILE
        if( NULL != a2dp_debug_file)
        {
         fwrite( ptr, 1, p->packetlen, a2dp_debug_file );
         if( !--a2dp_pack_count )
         {
          fclose( a2dp_debug_file );
          a2dp_debug_file = NULL;
         }
        }
#endif
        ec=write(((a2dp_packager_t *)(p->udata))->streamfd,ptr,
                                        p->packetlen);
        if( ec == -1 ) {
          DBG("Stream write error!\n");
	  writeError = 1;
	  break;
        } else {
          haveSent++;
          h->currentPos += packetPeriod;
	}
      }
    }

#ifdef DEBUG
	// store time for calculation of avg packet duration
    gettimeofday( &nowTime, NULL );
#endif

  }

exit_play:
  DBG( "A2DP player signalling EOS\n" );

	// signal eos to main thread
  pthread_mutex_lock(&(h->eosMutex));
  h->eos = 1;
  pthread_mutex_unlock(&(h->eosMutex));

  DBG( "A2DP player thread finished\n" );

  pthread_exit(0);
}

