/*
 * A2dpTypes.h
 * 
 * Brad Midgley
 * *************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#ifndef __A2DPTYPES_H
#define __A2DPTYPES_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
 
/* AVDTP structures */

/* packet components */

struct avdtp_header {
	uint8_t message_type:2;
	uint8_t packet_type:2;
	uint8_t transaction_label:4;
	uint8_t signal_id:6;
	uint8_t rfa0:2;
} __attribute__ ((packed));

struct avdtp_fragment_start_header {
	uint8_t message_type:2;
	uint8_t packet_type:2;
	uint8_t transaction_label:4;
    uint8_t number_of_fragments;
	uint8_t signal_id:6;
	uint8_t rfa0:2;
} __attribute__ ((packed));

struct avdtp_fragment_header {
	uint8_t message_type:2;
	uint8_t packet_type:2;
	uint8_t transaction_label:4;
} __attribute__ ((packed));

struct acp_seid_info {
	uint8_t rfa0:1;
	uint8_t inuse0:1;
	uint8_t acp_seid:6;
	uint8_t rfa2:3;
	uint8_t tsep:1;
	uint8_t media_type:4;
} __attribute__ ((packed));

struct sbc_codec_specific_elements {
	// a2dp p. 20
	uint8_t channel_mode:4;
	uint8_t frequency:4;
	uint8_t allocation_method:2;
	uint8_t subbands:2;
	uint8_t block_length:4;
	uint8_t min_bitpool;
	uint8_t max_bitpool;
} __attribute__ ((packed));

// allow this value to be overridden
#ifndef MAX_ADDITIONAL_CODEC
#define MAX_ADDITIONAL_CODEC 4
#endif

#define MAX_ADDITIONAL_CODEC_OCTETS (MAX_ADDITIONAL_CODEC*sizeof(struct acp_seid_info))

/* packets */

struct sepd_req {
	struct avdtp_header header;
} __attribute__ ((packed));

struct sepd_resp {
	struct avdtp_header header;
	struct acp_seid_info infos[1 + MAX_ADDITIONAL_CODEC];
} __attribute__ ((packed));

struct getcap_req {
	struct avdtp_header header;
	uint8_t rfa1:2;
	uint8_t acp_seid:6;
} __attribute__ ((packed));

struct getcap_resp {
	struct avdtp_header header;

    // Media transport capability
	uint8_t serv_cap;
	uint8_t serv_cap_len;

    // Media codec capability
	uint8_t cap_type;
	uint8_t length;
    
	uint8_t rfa1:4;
    uint8_t media_type:4;
	uint8_t media_codec_type;

	struct sbc_codec_specific_elements sbc_elements;

} __attribute__ ((packed));

typedef struct
{
	uint8_t serv_cap;
	uint8_t serv_cap_len;

	uint8_t cap_type;
	uint8_t length;
	uint8_t media_type;
	uint8_t media_codec_type;

	struct sbc_codec_specific_elements sbc_elements;
} __attribute__ ((packed)) service_capabilities;

struct set_config {
	struct avdtp_header header;

	uint8_t rfa0:2;
	uint8_t acp_seid:6;
	uint8_t rfa1:2;
	uint8_t int_seid:6;

    // Media transport capabilities
	uint8_t serv_cap;
	uint8_t serv_cap_len;

    // Media codec capabilities
	uint8_t cap_type;
	uint8_t length;
	uint8_t media_type;
	uint8_t media_codec_type;

	struct sbc_codec_specific_elements sbc_elements;

} __attribute__ ((packed));

struct set_config_resp {
	struct avdtp_header header;

	// only present for an error

	uint8_t serv_cat;
	uint8_t error_code;
} __attribute__ ((packed));

struct get_config {
	struct avdtp_header header;

	uint8_t rfa0:2;
	uint8_t acp_seid:6;
} __attribute__ ((packed));

struct get_config_resp {
	struct avdtp_header header;

    union
    {
     service_capabilities serv_cap;
 	 uint8_t error_code;
    };
} __attribute__ ((packed));

struct stream_reconfigure {
	struct avdtp_header header;

	uint8_t rfa0:2;
	uint8_t acp_seid:6;

    service_capabilities serv_cap;
} __attribute__ ((packed));

struct stream_reconfigure_resp {
	struct avdtp_header header;

	// only present for an error

	uint8_t serv_cat;
	uint8_t error_code;
} __attribute__ ((packed));

struct open_stream_cmd {
	struct avdtp_header header;
	uint8_t rfa0:2;
	uint8_t acp_seid:6;
} __attribute__ ((packed));

struct open_stream_rsp {
	struct avdtp_header header;

	// only present for an error

	uint8_t error;
} __attribute__ ((packed));

struct start_stream_cmd {
	struct avdtp_header header;
	uint8_t rfa0:2;
	uint8_t acp_seid:6;
} __attribute__ ((packed));

struct start_stream_rsp {
	struct avdtp_header header;

	// only present for an error

	uint8_t rfa0:2;
	uint8_t acp_seid:6;
	uint8_t error;
} __attribute__ ((packed));

struct close_stream_cmd {
	struct avdtp_header header;
	uint8_t rfa0:2;
	uint8_t acp_seid:6;
} __attribute__ ((packed));

struct close_stream_rsp {
	struct avdtp_header header;

	// only present for an error
	uint8_t error;
} __attribute__ ((packed));

struct abort_cmd {
	struct avdtp_header header;
	uint8_t rfa0:2;
	uint8_t acp_seid:6;
} __attribute__ ((packed));

struct abort_rsp {
	struct avdtp_header header;
} __attribute__ ((packed));

struct suspend_stream_cmd {
	struct avdtp_header header;
	uint8_t rfa0:2;
	uint8_t acp_seid:6;
} __attribute__ ((packed));

struct suspend_stream_rsp {
	struct avdtp_header header;

	// only present for an error
	uint8_t rfa0:2;
	uint8_t acp_seid:6;
	uint8_t error;
} __attribute__ ((packed));


// Generic command and response 
#define MAX_PACKET_SIZE   3000

typedef union
{
 struct avdtp_header       command_header;

 struct sepd_req           cmd_sepd_req;
 struct getcap_req         cmd_getcap_req;
 struct set_config         cmd_set_config;
 struct get_config         cmd_get_config;
 struct stream_reconfigure cmd_reconfigure;
 struct open_stream_cmd    cmd_open_stream;
 struct start_stream_cmd   cmd_start_stream;
 struct close_stream_cmd   cmd_close_stream;
 struct suspend_stream_cmd cmd_suspend_stream;
 struct abort_cmd          cmd_abort;
 uint8_t                   padding[ MAX_PACKET_SIZE ]; // Do not delete this !!!!!
} __attribute__ ((packed)) a2dp_generic_command_t;

typedef union 
{
 struct avdtp_header       responce_header;

 struct sepd_resp          resp_sepd_cmd;
 struct getcap_resp        resp_getcap_cmd;
 struct set_config_resp    resp_set_config_cmd;
 struct get_config_resp    resp_get_config_cmd;
 struct stream_reconfigure_resp resp_reconfigure_cmd;
 struct open_stream_rsp    resp_open_stream_cmd;
 struct start_stream_rsp   resp_start_stream_cmd;
 struct close_stream_rsp   resp_close_stream_cmd;
 struct suspend_stream_rsp resp_suspend_stream_cmd;
 struct abort_rsp          resp_abort_cmd;
 uint8_t                   padding[ MAX_PACKET_SIZE ]; // Do not delete this !!!!!
} __attribute__ ((packed)) a2dp_generic_responce_t;


// this is an rtp, not bluetooth header, so values are big endian
struct media_packet_header {
	uint8_t cc:4;
	uint8_t x:1;
	uint8_t p:1;
	uint8_t v:2;

	uint8_t pt:7;
	uint8_t m:1;

	uint16_t sequence_number;
	uint32_t timestamp;
	uint32_t ssrc;
	uint32_t csrc[0];
} __attribute__ ((packed));

struct media_payload_header {
	uint8_t frame_count:4;
	uint8_t rfa0:1;
	uint8_t is_last_fragment:1;
	uint8_t is_first_fragment:1;
	uint8_t is_fragmented:1;
} __attribute__ ((packed));

// Signal ID's
#define AVDTP_DISCOVER          0x01
#define AVDTP_GET_CAPABILITIES  0x02
#define AVDTP_SET_CONFIGURATION 0x03
#define AVDTP_GET_CONFIGURATION 0x04
#define AVDTP_RECONFIGURE       0x05
#define AVDTP_OPEN              0x06
#define AVDTP_START             0x07
#define AVDTP_CLOSE             0x08
#define AVDTP_SUSPEND           0x09
#define AVDTP_ABORT             0x0A

// Error codes
#define AVDTP_ERROR_BAD_HEADER_FORMAT         0x01 // All
#define AVDTP_ERROR_BAD_LENGTH                0x11 // All
#define AVDTP_ERROR_BAD_SEID                  0x12 // All
#define AVDTP_ERROR_SEP_IN_USE                0x13 // Set configuration
#define AVDTP_ERROR_SEP_NOT_IN_USE            0x14 // Reconfigure
#define AVDTP_ERROR_BAD_SERVICE_CATEGORY      0x17 // Set configuration, Reconfigure
#define AVDTP_ERROR_COMMAND_NOT_SUPPORTED     0x19 // All
#define AVDTP_ERROR_INVALID_CAPABILITIES      0x1A // Reconfigure
#define AVDTP_ERROR_BAD_MEDIA_TRANSPORT       0x23 // Set configuration, reconfigure
#define AVDTP_ERROR_UNSUPPORTED_CONFIGURATION 0x29 // Set configuration
#define AVDTP_ERROR_BAD_STATE                 0x31 // All

// PSM TYPES
#define CMD_PSM                  25

// End points
#define END_POINT_SOURCE         0x00
#define END_POINT_SINK           0x01
#define INT_SEID                 0x01 // Whatever

// Service categories
#define MEDIA_TRANSPORT_CATEGORY 0x01
#define MEDIA_CODEC_CATEGORY     0x07

#define SBC_MEDIA_CODEC_TYPE 0
#define AUDIO_MEDIA_TYPE 0

//Packet types
#define PACKET_TYPE_SINGLE 0
#define PACKET_TYPE_START 1
#define PACKET_TYPE_CONTINUE 2
#define PACKET_TYPE_END 3

//Message Types
#define MESSAGE_TYPE_COMMAND  0
#define MESSAGE_TYPE_RESERVED 1
#define MESSAGE_TYPE_ACCEPT   2
#define MESSAGE_TYPE_REJECT   3

#define MEDIA_PACKET_HEADER_LENGTH 14

// .au manipulation
// Is there a header that defines this stuff so we don't have to?

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define COMPOSE_ID(a,b,c,d)     ((a) | ((b)<<8) | ((c)<<16) | ((d)<<24))
#define LE_SHORT(v)             (v)
#define LE_INT(v)               (v)
#define BE_SHORT(v)             bswap_16(v)
#define BE_INT(v)               bswap_32(v)
#elif __BYTE_ORDER == __BIG_ENDIAN
#define COMPOSE_ID(a,b,c,d)     ((d) | ((c)<<8) | ((b)<<16) | ((a)<<24))
#define LE_SHORT(v)             bswap_16(v)
#define LE_INT(v)               bswap_32(v)
#define BE_SHORT(v)             (v)
#define BE_INT(v)               (v)
#else
#error "Wrong endian"
#endif
                                                                                                 
#define AU_MAGIC                COMPOSE_ID('.','s','n','d')
                                                                                                 
#define AU_FMT_ULAW             1
#define AU_FMT_LIN8             2
#define AU_FMT_LIN16            3

//Header format for the .snd files

struct au_header {
        uint32_t magic;         /* '.snd' */
        uint32_t hdr_size;      /* size of header (min 24) */
        uint32_t data_size;     /* size of data */
        uint32_t encoding;      /* see to AU_FMT_XXXX */
        uint32_t sample_rate;   /* sample rate */
        uint32_t channels;      /* number of channels (voices) */
};

#ifdef __cplusplus
}
#endif

#endif //  __A2DPTYPES_H__

/* EOF */
