/*      
 * File   : Encoder.h
 * Author : Koen Martens <koen.martens@tomtom.com>
 * Desc.  : Datatypes and function prototypes
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */  

#ifndef __ENCODER_H__
#define __ENCODER_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include "StreamBuffer.h"
#include "DynamicSBC.h"
//#include "sbc.h"

#ifdef DEBUG_PATH
#define SBCLIB "/mnt/sdcard/sbc.so"
#else
#define SBCLIB "/lib/sbc.so"
#endif

struct sbc_frame_header {
        uint8_t syncword:8;
        uint8_t subbands:1;
        uint8_t allocation_method:1;
        uint8_t channel_mode:2;
        uint8_t blocks:2;
        uint8_t sampling_frequency:2;
        uint8_t bitpool:8;
        uint8_t crc_check:8;
} __attribute__ ((packed));

typedef struct {
  void               *sbc_dl;
 // void               *udata;
  TSBCEncoder        *functions;
  int                (*framelength)(void *packager);	// func to get frame length
  int                (*byterate)(void *packager);	    // func to get byterate
  int                (*encode)(void *encoder,StreamBufferHandle *in,void *out);
  void               (*destroy)(void *); 
  
  // SBC settings:
  int              channels;                    // no. of channels (=2)
  int              channel_mode;                // (=2, stereo)
  int              rate;                        // frequency rate (=44100)
  int              subbands;                    // subbands (=8)
  int              blocks;                      // blocks (=16)
  int              bitpool;                     // bitpool (=64)
  int              allocation_method;

  // Thread control
  int              paused;
  int              resumed;
  int              empty_count;
  int              discarded;

  // SBC struct
  sbc_t            sbc;
  void*            empty_data; // For empty packet
} encoder_t;

encoder_t *InitEncoder( int aBitPool, char* aErrorText );

#define ENCODER_CONTINUE      0
#define ENCODER_EOS           1 //
#define ENCODER_UNDERRUN      2 //
#define ENCODER_TERM          3 //

#ifdef __cplusplus
}
#endif

#endif //  __ENCODER_H__

/* EOF */
