/*      
 * File   : Packager.h
 * Author : Koen Martens <koen.martens@tomtom.com>
 * Desc.  : Datastructures and function prototypes for packager abstraction
 *
 * Copyright (C) 2004,2005 TomTom BV <http://www.tomtom.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */  

#ifndef __PACKAGER_H
#define __PACKAGER_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include "StreamBuffer.h"
#include "Encoder.h"

typedef enum 
{ 
  KA2DP  = 0
} PackagerType;

typedef struct {
  void *udata;

  int  ( *connect )          ( void *packager, const char *dstaddr, char* aErrorText ); 
  int  ( *createpackage )    ( void *packager );
  void ( *destroy )          ( void *packager );
  int  ( *pause )            ( void *packager, char* aErrorText );
  int  ( *resume )           ( void *packager, char* aErrorText );
  int  ( *processpeerevents )( void *packager );    

  encoder_t *encoder;

  StreamBufferHandle *in;
  StreamBufferHandle *out;
  volatile int                 outFlushed;
  volatile int                 bytesWritten;
  volatile int		       totalFrames;
  volatile int                 missedFrames;
  volatile int                 lastFrame;
  volatile int		       packetlen;
  volatile int                 terminated;

  FILE                        *packetStream;

} packager_t;


#define PACKAGER_CONTINUE      0
#define PACKAGER_UNRECOVERABLE 1	
#define PACKAGER_EOS           2	// ???
#define PACKAGER_INVARG        3	
#define PACKAGER_STALL         4
#define PACKAGER_FLUSH         5	// not exposed above Packager


packager_t *InitPackager(
               encoder_t          *encoder,
               const char         *bdaddr,
               FILE               *pstream,
               char               *errorText );
void       *packagerProducer( void *args );
void        DonePackager( packager_t *packager );


#ifdef __cplusplus
}
#endif

#endif //  __PACKAGER_H__

/* EOF */
