#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <errno.h>

union semun {
  int val;
  struct semid_ds *buf;
  unsigned short int *array;
  struct seminfo *__buf;
};

static struct sembuf sysv_postop;
static struct sembuf sysv_waitop;
static struct sembuf sysv_tryop[2];

void sysv_init_ops(void) {
  sysv_postop.sem_num=0; sysv_postop.sem_op=1; sysv_postop.sem_flg=SEM_UNDO;
  sysv_waitop.sem_num=0; sysv_waitop.sem_op=-1; sysv_waitop.sem_flg=SEM_UNDO;

  sysv_tryop[0].sem_num=0; sysv_tryop[0].sem_op=0; sysv_tryop[0].sem_flg=IPC_NOWAIT;
  sysv_tryop[1].sem_num=0; sysv_tryop[1].sem_op=1; sysv_tryop[1].sem_flg=SEM_UNDO;
}

// note: does not prevent initialisation race conditions
int sysv_create_sem(const char *fname,int value) {
  int sem;
  key_t key;
  union semun arg;

  key=ftok(fname,666);
  if(key==-1) return -1;
 
  sem=semget(key,1,IPC_CREAT|0600);
  if(sem<0) return -1;

  arg.val=value;
  if(semctl(sem,0,SETVAL,arg)<0) return -1;

  sysv_init_ops();

  return sem;
}

// note: does not prevent initialisation race conditions
int sysv_init_sem(const char *fname) {
  int sem;
  key_t key;

  key=ftok(fname,666);
  if(key==-1) return -1;
 
  sem=semget(key,1,IPC_CREAT|0600);
  if(sem<0) return -1;

  sysv_init_ops();

  return sem;
}

// increase semaphore, 'unlock' only does not
// check whether it is actually locked so TAKE CARE
int sysv_unlock_sem(int sem) {
  return semop(sem,&sysv_postop,1);
}

// wait for semaphore to become >=1, and decrease
int sysv_lock_sem(int sem) {
  return semop(sem,&sysv_waitop,1);
}

// return 0 if already locked,
//        1 if locked succesfully
//       -1 if error occured
int sysv_trylock_sem(int sem) {
  int ec;

  do {
    ec=semop(sem,sysv_tryop,2);
    if(ec==-1) {
      if(errno==EAGAIN) return 0;
      if(errno!=EINTR) return -1;	// EINTR cannot happen (see man page)
    } else {
      return 1;
    }
  } while(1);
}

// return value:
//  0: locked
//  1: unlocked
// -1: error
int sysv_get_sem(int sem) {
  union semun arg;
  
  return semctl(sem,0,GETVAL,arg);
}

